<?php
/**
 * layout articles as a feed, with full content
 *
 * @see articles/articles.php
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
Class Layout_articles_as_contents extends Layout_interface {

	/**
	 * list articles
	 *
	 * @param resource the SQL result
	 * @return string the rendered text
	 *
	 * @see skins/layout.php
	**/
	function &layout(&$result) {
		global $context;

		// empty list
		if(!SQL::count($result)) {
			$output = array();
			return $output;
		}

		// load localized strings
		i18n::bind('articles');

		// we return an array of ($url => $attributes)
		$items = array();

		// process all items in the list
		while($item =& SQL::fetch($result)) {

			// reset the rendering engine between items
			if(is_callable(array('Codes', 'initialize')))
				Codes::initialize(Articles::get_url($item['id'], 'view', $item['nick_name']));

			// url to read the full article
			$url = $context['url_to_home'].$context['url_to_root'].Articles::get_url($item['id'], 'view', $item['nick_name']);

			// time of last update
			$time = strtotime($item['edit_date']);

			// the title as the label
			$label = ucfirst(Codes::beautify(strip_tags($item['title'], '<br><div><img><p><span>')));

			// the section
			$section = '';
			if($item['anchor'] && ($anchor = get_anchor($item['anchor'])))
				$section = ucfirst($anchor->get_title());

			// the icon to use
			$icon = '';
			if($item['thumbnail_url'])
				$icon = $item['thumbnail_url'];
			elseif($item['anchor'] && ($anchor = get_anchor($item['anchor'])))
				$icon = $anchor->get_thumbnail_url();
			if($icon)
				$icon = $context['url_to_home'].$context['url_to_home'].$icon;

			// the author(s) is an e-mail address, according to rss 2.0 spec
			$author = '';
			if(isset($item['create_address']))
				$author .= $item['create_address'];
			if(isset($item['create_name']) && trim($item['create_name']))
				$author .= ' ('.$item['create_name'].')';
			if(isset($item['edit_address']) && trim($item['edit_address']) && ($item['create_address'] != $item['edit_address'])) {
				if($author)
					$author .= ', ';
				$author .= $item['edit_address'];
				if(isset($item['edit_name']) && trim($item['edit_name']))
					$author .= ' ('.$item['edit_name'].')';
			}

			// some introductory text for this article
			include_once $context['path_to_root'].'articles/article.php';
			$article =& new Article();
			$article->load_by_content($item);
			$introduction = $article->get_teaser('teaser');

			// warns on restricted access
			if($item['active'] != 'Y')
				$introduction = '['.i18n::c('Restricted to members').'] '.$introduction;

			// fix references
			$introduction = preg_replace('/"\//', '"'.$context['url_to_home'].'/', $introduction);

			// the article content
			$description = '';

			if($item['introduction'])
				$description .= Skin::build_block($item['introduction'], 'introduction');
			$description .= Codes::beautify(preg_replace('/\[(toc|toq)\]\s+/i', '', $item['description']), $item['options']);

			// cap the number of words
//			$description = Skin::cap($description, 300);

			// fix references
			$description = preg_replace('/"\//', '"'.$context['url_to_home'].'/', $description);

			// other rss fields
			$extensions = array();

			// url for comments
			$extensions[] = '<comments>'.$context['url_to_home'].$context['url_to_root'].Articles::get_url($item['id']).'#comments</comments>';

			// count comments
			$comment_count = 0;
			include_once $context['path_to_root'].'comments/comments.php';
			if($stat =& Comments::stat_for_anchor('article:'.$item['id']))
				$comment_count = $stat['count'];
			$extensions[] = '<slash:comments>'.$comment_count."</slash:comments>";

			// the comment post url
			$extensions[] = '<wfw:comment>'.$context['url_to_home'].$context['url_to_root'].Comments::get_url('article:'.$item['id'], 'service.comment')."</wfw:comment>";

			// the comment Rss url
			$extensions[] = '<wfw:commentRss>'.$context['url_to_home'].$context['url_to_root'].Comments::get_url('article:'.$item['id'], 'feed')."</wfw:commentRss>";

			// the trackback url
			$extensions[] = '<trackback:ping>'.$context['url_to_home'].$context['url_to_root'].'links/trackback.php?anchor=article:'.$item['id']."</trackback:ping>"; // no trackback:about;

			// list all components for this item
			$items[$url] = array($time, $label, $author, $section, $icon, $introduction, $description, $extensions);

		}

		// end of processing
		SQL::free($result);
		return $items;
	}

}

?>