<?php
/**
 * layout articles as an index page of Yahoo!
 *
 * With this layout up to three sub-items are listed as well.
 * These can be either files or links, depending of relative availability of both kind of items.
 *
 * @see articles/articles.php
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
Class Layout_articles_as_yahoo extends Layout_interface {

	/**
	 * list articles
	 *
	 * @param resource the SQL result
	 * @return string the rendered text
	 *
	 * @see skins/layout.php
	**/
	function &layout(&$result) {
		global $context;

		// empty list
		if(!SQL::count($result)) {
			$output = array();
			return $output;
		}

		// load localized strings
		i18n::bind('articles');

		// the number of related items to display
		if(!defined('YAHOO_LIST_SIZE'))
			define('YAHOO_LIST_SIZE', 3);

		// flag articles updated recently
		if($context['site_revisit_after'] < 1)
			$context['site_revisit_after'] = 2;
		$dead_line = strftime('%Y-%m-%d %H:%M:%S', mktime(0,0,0,date("m"),date("d")-$context['site_revisit_after'],date("Y")));
		$now = strftime('%Y-%m-%d %H:%M:%S');

		// we return an array of ($url => $attributes)
		$items = array();

		// process all items in the list
		include_once $context['path_to_root'].'comments/comments.php';
		include_once $context['path_to_root'].'files/files.php';
		include_once $context['path_to_root'].'links/links.php';
		include_once $context['path_to_root'].'overlays/overlay.php';
		while($item =& SQL::fetch($result)) {

			// the url to view this item
			$url = Articles::get_url($item['id'], 'view', $item['nick_name']);

			// reset the rendering engine between items
			Codes::initialize($url);

			// get the main anchor
			$anchor = get_anchor($item['anchor']);

			// get the related overlay, if any
			$overlay = Overlay::load($item);

			// initialize variables
			$prefix = $label = $suffix = $icon = '';

			// flag articles that are dead, or created or updated very recently
			if(($item['expiry_date'] > NULL_DATE) && ($item['expiry_date'] <= $now))
				$prefix .= EXPIRED_FLAG;
			elseif($item['create_date'] >= $dead_line)
				$suffix .= NEW_FLAG;
			elseif($item['edit_date'] >= $dead_line)
				$suffix .= UPDATED_FLAG;

			// signal articles to be published
			if(($item['publish_date'] <= NULL_DATE) || ($item['publish_date'] > gmstrftime('%Y-%m-%d %H:%M:%S')))
				$prefix .= DRAFT_FLAG;

			// signal locked articles
			if(isset($item['locked']) && ($item['locked'] == 'Y'))
				$prefix .= LOCKED_FLAG;

			// signal restricted and private articles
			if($item['active'] == 'N')
				$prefix .= PRIVATE_FLAG;
			elseif($item['active'] == 'R')
				$prefix .= RESTRICTED_FLAG;

			// details
			$details = array();

			// count related sub-elements
			$related_count = 0;

			// info on related files
			$stats = Files::stat_for_anchor('article:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;file', '%d&nbsp;files', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related links
			$stats = Links::stat_for_anchor('article:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;link', '%d&nbsp;links', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related comments
			$stats = Comments::stat_for_anchor('article:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;comment', '%d&nbsp;comments', $stats['count']), $stats['count']);

			// rank, for associates
			if(($item['rank'] != 10000) && Surfer::is_empowered())
				$details[] = '{'.$item['rank'].'}';

			// introduction
			if($item['introduction'])
				$suffix .= ' '.strip_tags(Codes::beautify(trim($item['introduction'])), '<a><br><img><li><ol><ul>');

			// append details to the suffix
			if(count($details))
				$suffix .= ' <span class="details">('.implode(', ', $details).')</span>';

			// add a head list of related links
			$details = array();

			// add related files if necessary
			if((count($details) < YAHOO_LIST_SIZE) && ($related = Files::list_by_date_for_anchor('article:'.$item['id'], 0, YAHOO_LIST_SIZE - count($details), 'compact'))) {
				foreach($related as $url => $label) {
					$sub_prefix = $sub_suffix = $sub_hover = '';
					if(is_array($label)) {
						$sub_prefix = $label[0];
						$sub_suffix = $label[2];
						if(@$label[5])
							$sub_hover = $label[5];
						$label = $label[1];
					}
					$details[] = $sub_prefix.Skin::build_link($url, $label, 'basic', $sub_hover).$sub_suffix;
				}
			}

			// add related links if necessary
			if((count($details) < YAHOO_LIST_SIZE) && ($related = Links::list_by_date_for_anchor('article:'.$item['id'], 0, YAHOO_LIST_SIZE - count($details), 'compact'))) {
				foreach($related as $url => $label) {
					$sub_prefix = $sub_suffix = $sub_hover = '';
					if(is_array($label)) {
						$sub_prefix = $label[0];
						$sub_suffix = $label[2];
						if(@$label[5])
							$sub_hover = $label[5];
						$label = $label[1];
					}
					$details[] = $sub_prefix.Skin::build_link($url, $label, 'basic', $sub_hover).$sub_suffix;
				}
			}

			// give me more
			if(count($details) && ($related_count > YAHOO_LIST_SIZE))
				$details[] = Skin::build_link(Articles::get_url($item['id'], 'view', $item['nick_name']), i18n::s('More').MORE_IMG, 'more', i18n::s('This article contains other information of interest'));

			// layout details
			if(count($details))
				$suffix .= BR.'<span class="small">'.YAHOO_ITEM_PREFIX.implode(YAHOO_ITEM_SUFFIX.YAHOO_ITEM_PREFIX, $details).YAHOO_ITEM_SUFFIX."</span>\n";

			// use the title to label the link
			if(is_object($overlay) && is_callable(array($overlay, 'get_live_title')))
				$label = $overlay->get_live_title($item);
			else
				$label = Skin::strip($item['title'], 50);

			// put the actual icon in the left column
			if(isset($item['thumbnail_url']))
				$icon = $item['thumbnail_url'];

			// some hovering title for this article
			$hover = i18n::s('Read the page');

			// list all components for this item --use basic link style to avoid prefix or suffix images, if any
			$items[$url] = array($prefix, $label, $suffix, 'basic', $icon, $hover);

		}

		// end of processing
		SQL::free($result);
		return Skin::build_list($items, '2-columns');
	}

}

?>