<?php
/**
 * the control panel for this web server
 *
 * The control panel has several sections:
 *
 * - Commands are provided for system administration (e.g., database backup), but only to associates.
 *
 * - A set of commands related to content management are provided as well.
 *
 * - Several commands are available to associates to the several configuration and setup panels.
 * This means that every YACS server can be configured directly from the control panel.
 *
 * - The control panel gives an overview of the database content; this section may be accessed by anybody.
 * Useful to check the number of pages, images and files that have been posted.
 * The total size of images and files is provided as well.
 *
 * - Software versions are provided as well, including YACS, PHP, MySQL and Apache, but not to software robots nor crawlers.
 *
 * - Additional tools are provided to authenticated members, including bookmarklets.
 *
 * - Also, a list of modules is displayed.
 *
 * The control panel has a hook to integrate extended configuration panels:
 * - id: 'control/index.php#configure'
 * - type: 'link'
 *
 * It also has a hook to list extra modules:
 * - id: 'control/index.php#modules'
 * - type: 'link'
 *
 * @see control/scan.php
 *
 * Actually the control panel is not always displayed.
 * To ease the setup tasks this script is working like an assistant:
 *
 * - If the main configuration file [code]parameters/control.include.php[/code] is absent,
 * propose to switch either to [script]setup.php[/script] or to [script]control/configure.php[/script]
 *
 * - If there is no database, propose to switch to [script]control/configure.php[/script]
 *
 * - If the configuration file [code]parameters/hooks.include.php[/code] is absent,
 * propose to switch to [script]control/scan.php[/script]
 *
 * - If there is no tables in database, propose to switch to [script]control/setup.php[/script]
 *
 * - If the users table is empty, propose to switch to [script]control/populate.php[/script]
 *
 * - If the skin configuration file [code]parameters/skins.include.php[/code] is absent,
 * propose to switch to [script]skins/configure.php[/script]
 *
 * - If there is no switch file, jump to the [script]setup.php[/script] page
 *
 * - Else actually display the control panel
 *
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author Christophe Battarel [email]christophe.battarel@altairis.fr[/email]
 * @author GnapZ
 * @tester Agnes
 * @tester FabriceV
 * @tester Lucrecius
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// include some libraries
include_once '../shared/global.php';
include_once '../sections/sections.php';
include_once '../articles/articles.php';

// load localized strings
i18n::bind('control');

// load the skin
load_skin('control');

// no parameters configured yet
if(!file_exists('../parameters/control.include.php')) {

	// title
	$context['page_title'] = i18n::s('Please configure your server');

	// splash screen
	$context['text'] .= '<p>'.i18n::s('No configuration file has been found. If you are installing a brand new server, follow the link below to create one.')."</p>\n";

	// link to the installation page
	$context['text'] .= '<p><a href="../setup.php">'.i18n::s('Jump to the installation page')."</a></p>\n";

	// splash screen
	$context['text'] .= '<p>'.i18n::s('Else follow the link below to load the configuration form.')."</p>\n";

	// link to the configuration page
	$context['text'] .= '<p><a href="configure.php">'.i18n::s('Jump to the configuration page')."</a></p>\n";

// no access to the database server yet
} elseif(!isset($context['database']) || !$context['database']
		|| !isset($context['connection']) || !$context['connection']) {

	// title
	$context['page_title'] = i18n::s('No access to the database server');

	// splash screen
	$context['text'] .= '<p>'.i18n::s('Impossible to connect to the database server mentioned in your configuration file. Please follow the link to check connection parameters.')."</p>\n";

	// link to the configuration page
	$context['text'] .= '<p><a href="configure.php">'.i18n::s('Jump to the configuration page')."</a></p>\n";

} else {

	// try to create a database if it does not exist
	if(!mysql_select_db($context['database'], $context['connection']) && is_callable('mysql_create_db'))
		mysql_create_db($context['database'], $context['connection']);

	// still no database
	if(!mysql_select_db($context['database'], $context['connection'])) {

		// title
		$context['page_title'] = i18n::s('Please create a database');

		// splash screen
		$context['text'] .= '<p>'.i18n::s('Impossible to access the database mentioned in your configuration file. Please create a database, or follow the link to change the configuration file.')."</p>\n";

		// link to the configuration page
		$context['text'] .= '<p><a href="configure.php">'.i18n::s('Go to the configuration page to change database parameters')."</a></p>\n";

	// no hooks found yet
	} elseif(!file_exists('../parameters/hooks.include.php')) {

		// title
		$context['page_title'] = i18n::s('Please configure software extensions');

		// splash screen
		$context['text'] .= '<p>'.i18n::s('No configuration file for extensions has been found. If you are installing a brand new server, follow the link to create one.')."</p>\n";

		// link to the scan page
		$context['text'] .= '<p><a href="scan.php">'.i18n::s('Look for software extensions')."</a></p>\n";

	// no tables
	} elseif((!$result = mysql_list_tables($context['database'], $context['connection']))
		|| (SQL::count($result) < 5)) {

		// title
		$context['page_title'] = i18n::s('Please create tables');

		// splash screen
		$context['text'] .= '<p>'.i18n::s('The database is currently empty. Please follow the link to create tables of the database.')."</p>\n";

		// link to the setup page
		$context['text'] .= '<p><a href="setup.php">'.i18n::s('Create tables in the database')."</a></p>\n";

	} else {

		// the user table is empty
		$query = "SELECT count(*) FROM ".SQL::table_name('users');
		$count = 0;
		if($result =& SQL::query($query, TRUE, $context['users_connection'])) {
			$row = SQL::fetch_row($result);
			$count = $row[0];
		}
		if(!$result || ($count == 0)) {

			// title
			$context['page_title'] = i18n::s('Please populate tables');

			// splash screen
			$context['text'] .= '<p>'.i18n::s('The user table is currently empty. Please follow the link to populate the database.')."</p>\n";

			// link to the populate page
			$context['text'] .= '<p><a href="populate.php">'.i18n::s('Jump to the populate page')."</a></p>\n";

		// no parameters configured yet for the skin
		} elseif(!file_exists('../parameters/skins.include.php')) {

			// title
			$context['page_title'] = i18n::s('Please configure the skin of your server');

			// splash screen
			$context['text'] .= '<p>'.i18n::s('No configuration file has been found for the skin of your server. If you are installing a brand new server, follow the link to create one.')."</p>\n";

			// link to the configuration page
			$context['text'] .= '<p><a href="../skins/configure.php">'.i18n::s('Jump to the skin configuration page')."</a></p>\n";

		// end of verifications
		} else {

			// the title of the page
			$context['page_title'] = i18n::s('Control Panel');

			// server is closed
			if(file_exists($context['path_to_root'].'parameters/switch.off')) {

				// title
				Skin::error(i18n::s('The server is currently switched off. All users are redirected to the closed page.'));

				// link to the switch page
				if(Surfer::is_associate())
					$context['text'] = '<p style="text-decoration: blink;"><a href="switch.php?op=on">'.i18n::s('Switch the server on again').'</a></p>';

			// there is no switch file, redirect to the setup assistant
			} elseif(!file_exists($context['path_to_root'].'parameters/switch.on'))
				Safe::redirect($context['url_to_home'].$context['url_to_root'].'setup.php');

			// server is running on demonstration mode
			if(file_exists($context['path_to_root'].'parameters/demo.flag'))
				Skin::error(i18n::s('The server is running in demonstration mode, and restrictions apply, even to associates.'));

			//
			// commands for associates
			//

			// menu bar, and related help, for associates
			if(Surfer::is_associate()) {

				// links to system commands
				$context['text'] .= Skin::build_block(i18n::s('System management'), 'title');

				// commands, and related help
				$context['page_menu'] = array();
				$help = array();

				$context['page_menu'] = array_merge($context['page_menu'], array('control/backup.php' => i18n::s('Backup/Restore')));
				$help[] = sprintf(i18n::s('%s - the safety tool; also useful to submit bulk SQL statements'), '<a href="backup.php">'.i18n::s('Backup/Restore').'</a>');

				$context['page_menu'] = array_merge($context['page_menu'], array('control/setup.php' => i18n::s('Maintenance')));
				$help[] = sprintf(i18n::s('%s - check the database structure; also optimize data tables and update index'), '<a href="setup.php">'.i18n::s('Maintenance').'</a>');

				$context['page_menu'] = array_merge($context['page_menu'], array('control/purge.php' => i18n::s('Purge')));
				$help[] = sprintf(i18n::s('%s - clear the cache or delete what can be safely deleted'), '<a href="purge.php">'.i18n::s('Purge').'</a>');

				$context['page_menu'] = array_merge($context['page_menu'], array('control/scan.php' => i18n::s('Extensions')));
				$help[] = sprintf(i18n::s('%s - review and setup YACS hooking pieces of software'), '<a href="scan.php">'.i18n::s('Extensions').'</a>');

				$context['page_menu'] = array_merge($context['page_menu'], array('control/switch.php' => i18n::s('Switch')));
				$help[] = sprintf(i18n::s('%s - shut the server down, or fire it up again'), '<a href="switch.php">'.i18n::s('Switch').'</a>');

				// signal scripts to run once, if any
				if(Safe::glob($context['path_to_root'].'scripts/run_once/*.php') !== FALSE) {
					$context['page_menu'] = array_merge($context['page_menu'], array( 'scripts/run_once.php' => i18n::s('Run once') ));
					$help[] = sprintf(i18n::s('%s - some upgrades are waiting for execution'), '<a href="../scripts/run_once.php">'.i18n::s('Run once').'</a>');
				}

				// refresh if there is a reference repository
				if(file_exists($context['path_to_root'].'scripts/reference/footprints.php')) {
					$context['page_menu'] = array_merge($context['page_menu'], array('scripts/' => i18n::s('Scripts')));
					$help[] = sprintf(i18n::s('%s - this server shares a reference store for remote updates'), '<a href="../scripts/">'.i18n::s('Scripts').'</a>');

				// upgrade if there is no reference repository
				} else {
					$context['page_menu'] = array_merge($context['page_menu'], array('scripts/' => i18n::s('Upgrade')));
					$help[] = sprintf(i18n::s('%s - check for software updates'), '<a href="../scripts/">'.i18n::s('Upgrade').'</a>');
				}

				$context['page_menu'] = array_merge($context['page_menu'], array('control/info.php' => i18n::s('System information')));
				$help[] = sprintf(i18n::s('%s - various internal reports'), '<a href="info.php">'.i18n::s('System information').'</a>');

				// list of commands
				$context['text'] .= '<ul>'
					.'<li>'.join('</li><li>', $help).'</li>'
					.'</ul>';

			}

			//
			// content management
			//

			// display a list of content modules
			if(class_exists('Skin')) {

				// links to configuration panels
				$context['text'] .= Skin::build_block(i18n::s('Content management'), 'title');

				// the list of modules
				$context['text'] .= '<ul>';

				// content assistant
				if(Surfer::is_associate())
					$context['text'] .= '<li>'.sprintf(i18n::s('%s - create blogs, wikis, forums, and more'), Skin::build_link('control/populate.php', i18n::s('Content Assistant'), 'basic')).'</li>';

				// review queue
				if(Surfer::is_associate())
					$context['text'] .= '<li>'.sprintf(i18n::s('%s - articles waiting for publication, pending requests'), Skin::build_link('articles/review.php', i18n::s('Review queue'), 'basic')).'</li>';

				// collections
				$context['text'] .= '<li>'.sprintf(i18n::s('%s - shared directories and files'), Skin::build_link('collections/', i18n::s('Collections'), 'basic')).'</li>';

				// letters
				if(Surfer::is_associate() && isset($context['with_email']) && ($context['with_email'] == 'Y'))
					$context['text'] .= '<li>'.sprintf(i18n::s('%s - broadcast digests and announcements by e-mail'), Skin::build_link('letters/', i18n::s('Letters'), 'basic')).'</li>';

				// available feeds
				$context['text'] .= '<li>'.sprintf(i18n::s('%s - all feeds available (RSS, ATOM)'), Skin::build_link('feeds/', i18n::s('Syndication'), 'basic')).'</li>';

				// web services
				$context['text'] .= '<li>'.sprintf(i18n::s('%s - connect remote computers through XML-RPC or REST'), Skin::build_link('services/', i18n::s('Web services'), 'basic')).'</li>';

				// usage information
				if(Surfer::is_associate())
					$context['text'] .= '<li>'.sprintf(i18n::s('%s - at the moment, only for people that are migrating from phpwebsite'), Skin::build_link('control/import.php', i18n::s('Import'), 'basic')).'</li>';

				// usage information
				if(Surfer::is_associate())
					$context['text'] .= '<li>'.sprintf(i18n::s('%s - import articles from a csv file'), Skin::build_link('control/import_csv.php', i18n::s('Import CSV'), 'basic')).'</li>';

				// usage information
				if(Surfer::is_associate())
					$context['text'] .= '<li>'.sprintf(i18n::s('%s - learn about your visitors'), Skin::build_link('agents/', i18n::s('Usage information'), 'basic')).'</li>';

				// end of the list
				$context['text'] .= "</ul>\n";
			}

			//
			// configuration panels, for associates
			//

			// display a list of configuration panels for associates, if a skin has been defined
			if(Surfer::is_associate() && class_exists('Skin')) {

				// links to configuration panels
				$context['text'] .= Skin::build_block(i18n::s('Configuration panels'), 'title');

				// the list of modules
				$context['text'] .= '<ul>';

				// configuration scripts that are part of the core
				$context['text'] .= '<li>'.sprintf(i18n::s('%s - configure database, security and other essential parameters'), Skin::build_link('control/configure.php', i18n::s('System'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - define permissions given to users'), Skin::build_link('users/configure.php', i18n::s('Users'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - select and test skins available at this server'), Skin::build_link('skins/', i18n::s('Skins'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - build your front page, change meta-information, etc.'), Skin::build_link('skins/configure.php', i18n::s('Rendering'), 'basic')).'</li>';

				if(isset($context['home_with_internal_news']) && ($context['home_with_internal_news'] == 'Y'))
					$context['text'] .= '<li>'.sprintf(i18n::s('%s - to change rendering of dynamic Flash objects'), Skin::build_link('feeds/flash/configure.php', i18n::s('Flash'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - enhance information provided through RSS'), Skin::build_link('feeds/configure.php', i18n::s('Syndication'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - to share and stream existing directories and files'), Skin::build_link('collections/configure.php', i18n::s('Collections'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - to add extensions and to make uploaded files available from FTP'), Skin::build_link('files/configure.php', i18n::s('Files'), 'basic')).'</li>';

				if(isset($context['with_email']) && ($context['with_email'] == 'Y'))
					$context['text'] .= '<li>'.sprintf(i18n::s('%s - change the template used for newsletters'), Skin::build_link('letters/configure.php', i18n::s('Letters'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - change parameters for back-end services'), Skin::build_link('services/configure.php', i18n::s('Web services'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - ban spamming hosts'), Skin::build_link('servers/configure.php', i18n::s('Servers'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - options for background processing'), Skin::build_link('agents/configure.php', i18n::s('Agents'), 'basic')).'</li>';

				$context['text'] .= '<li>'.sprintf(i18n::s('%s - change the reference server used for software updates'), Skin::build_link('scripts/configure.php', i18n::s('Upgrade'), 'basic')).'</li>';

				// the hook for the control panels
				if(is_callable(array('Hooks', 'link_scripts')))
					$context['text'] .= Hooks::link_scripts('control/#configure', 'bullets');

				// end of the list
				$context['text'] .= "</ul>\n";
			}

			//
			// database overview
			//

			// display a database overview if a skin has been defined
			if(class_exists('Skin')) {

				$context['text'] .= Skin::build_block(i18n::s('Database overview'), 'title');

				// use a neat table for the layout
				$context['text'] .= Skin::table_prefix('');
				if($context['language'] == 'fr')
					$context['text'] .= Skin::table_row(array('Table', 'Enregistrements', 'center=Premier enregistrement', 'center=Dernier enregistrement'), 'header');
				else
					$context['text'] .= Skin::table_row(array('Table', 'Records', 'center=First record', 'center=Last record'), 'header');
				$lines = 2;

				// articles
				if($row = SQL::table_stat('articles')) {
					$cells = array();
					$cells[] = Skin::build_link('articles/', i18n::s('Articles'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Articles'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// images
				include_once '../images/images.php';
				if($stats = Images::stat()) {
					$cells = array();
					$size = '';
					if($stats['total_size'])
						$size = ' ('.Skin::build_number($stats['total_size']).')';
					$cells[] = Skin::build_link('images/', i18n::s('Images'), 'shortcut').$size;
					$cells[] = 'center='.$stats['count'];
					$cells[] = 'center='.($stats['oldest_date']?Skin::build_date($stats['oldest_date']):'--');
					$cells[] = 'center='.($stats['newest_date']?Skin::build_date($stats['newest_date']):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Images'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// tables
				if($row = SQL::table_stat('tables')) {
					$cells = array();
					$cells[] = Skin::build_link('tables/', i18n::s('Tables'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Tables'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// files
				include_once '../files/files.php';
				if($stats = Files::stat()) {
					$cells = array();
					$size = '';
					if($stats['total_size'])
						$size = ' ('.Skin::build_number($stats['total_size']).')';
					$cells[] = Skin::build_link('files/', i18n::s('Files'), 'shortcut').$size;
					$cells[] = 'center='.$stats['count'];
					$cells[] = 'center='.($stats['oldest_date']?Skin::build_date($stats['oldest_date']):'--');
					$cells[] = 'center='.($stats['newest_date']?Skin::build_date($stats['newest_date']):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Files'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// links
				if($row = SQL::table_stat('links')) {
					$cells = array();
					$cells[] = Skin::build_link('links/', i18n::s('Links'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Links'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// locations
				if($row = SQL::table_stat('locations')) {
					$cells = array();
					$cells[] = Skin::build_link('locations/', i18n::s('Locations'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Locations'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// comments
				if($row = SQL::table_stat('comments')) {
					$cells = array();
					$cells[] = sprintf(i18n::s('Comments in %s'), Skin::build_link('comments/', i18n::s('threads'), 'shortcut'));
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Comments'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// decisions
				if($row = SQL::table_stat('decisions')) {
					$cells = array();
					$cells[] = Skin::build_link('decisions/', i18n::s('Decisions'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Decisions'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// categories
				if($row = SQL::table_stat('categories')) {
					$cells = array();
					$cells[] = Skin::build_link('categories/', i18n::s('Categories'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Categories'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// sections
				if($row = SQL::table_stat('sections')) {
					$cells = array();
					$cells[] = Skin::build_link('sections/', i18n::s('Sections'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Sections'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// users
				if($row = SQL::table_stat('users')) {
					$cells = array();
					$cells[] = Skin::build_link('users/', i18n::s('Users'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Users'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// actions
				if($row = SQL::table_stat('actions')) {
					$cells = array();
					$cells[] = Skin::build_link('actions/', i18n::s('Actions'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Actions'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// dates
				if($row = SQL::table_stat('dates')) {
					$cells = array();
					$cells[] = Skin::build_link('dates/', i18n::s('Dates'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Dates'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// servers
				if($row = SQL::table_stat('servers')) {
					$cells = array();
					$cells[] = Skin::build_link('servers/', i18n::s('Servers'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Servers'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// referrals
				include_once '../agents/referrals.php';
				if($stats = Referrals::stat()) {
					$cells = array();
					$cells[] = i18n::s('Referrals');
					$cells[] = 'center='.$stats['count'];
					$cells[] = 'center=--';
					$cells[] = 'center=--';
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Referrals'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// counters
				include_once '../agents/browsers.php';
				if($stats = Browsers::stat()) {
					$cells = array();
					$cells[] = i18n::s('Counters');
					$cells[] = 'center='.$stats['count'];
					$cells[] = 'center=--';
					$cells[] = 'center=--';
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Counters'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// profiles
				include_once '../agents/profiles.php';
				if($stats = Profiles::stat()) {
					$cells = array();
					$cells[] = i18n::s('Profiles');
					$cells[] = 'center='.$stats['count'];
					$cells[] = 'center=--';
					$cells[] = 'center=--';
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Profiles'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// versions
				if($row = SQL::table_stat('versions')) {
					$cells = array();
					$cells[] = i18n::s('Versions');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Versions'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// members
				if($row = SQL::table_stat('members')) {
					$cells = array();
					$cells[] = i18n::s('Members of...');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Members of...'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// values
				if($row = SQL::table_stat('values')) {
					$cells = array();
					$cells[] = i18n::s('Values');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Values'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// cache
				if($row = SQL::table_stat('cache')) {
					$cells = array();
					$cells[] = i18n::s('Cache');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('Cache'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// the php documentation
				if($row = SQL::table_stat('phpdoc')) {
					$cells = array();
					$cells[] = Skin::build_link('scripts/', i18n::s('PHP documentation'), 'shortcut');
					$cells[] = 'center='.$row[0];
					$cells[] = 'center='.($row[1]?Skin::build_date($row[1]):'--');
					$cells[] = 'center='.($row[2]?Skin::build_date($row[2]):'--');
					$context['text'] .= Skin::table_row($cells, $lines++);
				} else
					$context['text'] .= Skin::table_row(array(i18n::s('PHP documentation'), i18n::s('unknown or empty table'), ' ', ' '), $lines++);

				// end of the table
				$context['text'] .= Skin::table_suffix();

			}

			//
			// system overview
			//

			// display a database overview if not a crawler, and if a skin has been defined
			if(!Surfer::is_crawler() && class_exists('Skin')) {

				$context['text'] .= Skin::build_block(i18n::s('System overview'), 'title');

				// use a neat table for the layout
				$context['text'] .= Skin::table_prefix('');
				$lines = 1;

				// yacs version
				if(!isset($generation['version']))
					Safe::load('footprints.php');						// initial archive, or current version
				if(!isset($generation['version']))
					Safe::load('scripts/reference/footprints.php');		// on-going development
				if(!isset($generation['version']))
					Safe::load('scripts/staging/footprints.php');		// last update
				$cells = array();
				$cells[] = Skin::build_link('http://www.yetanothercommunitysystem.com/', 'YACS');
				if(isset($generation['version']))
					$cells[] = $generation['version'].', '.$generation['date'].', '.$generation['server'];
				else
					$cells[] = '< 6.3';
				$context['text'] .= Skin::table_row($cells, $lines++);

				// php version
				$cells = array();
				$cells[] = Skin::build_link('http://www.php.net/', 'PHP');
				$cells[] = phpversion();
				$context['text'] .= Skin::table_row($cells, $lines++);

				// MySQL version
				if(is_callable('mysql_get_server_info')) {
					$cells = array();
					$cells[] = Skin::build_link('http://www.mysql.com/', 'MySQL');
					$cells[] = mysql_get_server_info();
					$context['text'] .= Skin::table_row($cells, $lines++);
				}

				// Apache version
				if(is_callable('apache_get_version')) {
					$cells = array();
					$cells[] = Skin::build_link('http://www.apache.org/', 'Apache');
					$cells[] = apache_get_version();
					$context['text'] .= Skin::table_row($cells, $lines++);
				}

				// time shift
				$cells = array();
				$cells[] = i18n::s('Server time zone');
				$cells[] = sprintf('UTC %s%s %s', ($context['gmt_offset'] > 0)?'+':'', $context['gmt_offset'], i18n::ns('hour', 'hours', abs($context['gmt_offset'])));
				$context['text'] .= Skin::table_row($cells, $lines++);

				// memory usage
				if(is_callable('memory_get_usage')) {
					$cells = array();
					$cells[] = i18n::s('Memory');
					$cells[] = memory_get_usage();
					$context['text'] .= Skin::table_row($cells, $lines++);
				}

				// end of the table
				$context['text'] .= Skin::table_suffix();

			}

			//
			// extend the toolbox
			//

			// members can use additional tools
			if(Surfer::is_member()) {
				$context['text'] .= Skin::build_block(i18n::s('Blogging tools'), 'title');

				// introduce bookmarklets
				$context['text'] .= '<p>'.i18n::s('To install following bookmarklets, right-click over them and add them to your bookmarks or favorites. Then recall them at any time while browsing the Internet, to add content to this site.').'</p>'."\n".'<ul>';

				// the blogging bookmarklet uses YACS codes
				$bookmarklet = "javascript:function findFrame(f){var i;try{isThere=f.document.selection.createRange().text;}catch(e){isThere='';}if(isThere==''){for(i=0;i&lt;f.frames.length;i++){findFrame(f.frames[i]);}}else{s=isThere}return s}"
					."var s='';"
					."d=document;"
					."s=d.selection?findFrame(window):window.getSelection();"
					."window.location='".$context['url_to_home'].$context['url_to_root']."articles/edit.php?"
						."title='+escape(d.title)+'"
						."&amp;text='+escape('%22'+s+'%22%5Bnl]-- %5Blink='+d.title+']'+d.location+'%5B/link]')+'"
						."&amp;source='+escape(d.location);";
				$context['text'] .= '<li><a href="'.$bookmarklet.'">'.sprintf(i18n::s('Blog at %s'), $context['site_name']).'</a></li>'."\n";

				// the bookmarking bookmarklet
				$bookmarklet = "javascript:function findFrame(f){var i;try{isThere=f.document.selection.createRange().text;}catch(e){isThere='';}if(isThere==''){for(i=0;i&lt;f.frames.length;i++){findFrame(f.frames[i]);}}else{s=isThere}return s}"
					."var s='';"
					."d=document;"
					."s=d.selection?findFrame(window):window.getSelection();"
					."window.location='".$context['url_to_home'].$context['url_to_root']."links/edit.php?"
						."link='+escape(d.location)+'"
						."&amp;title='+escape(d.title)+'"
						."&amp;text='+escape(s);";
				$context['text'] .= '<li><a href="'.$bookmarklet.'">'.sprintf(i18n::s('Bookmark at %s'), $context['site_name']).'</a></li>'."\n";

				// end of bookmarklets
				$context['text'] .= '</ul>'."\n";

				// the command to add a side panel
				$context['text'] .= '<p>'.sprintf(i18n::s('If your browser supports side panels and javascript, click on the following link to %s'), '<a onclick="javascript:addSidePanel()">'.i18n::s('add a blogging panel').'</a>.').'</p>'."\n";

				// the actual javascript code to add a panel
				$context['page_footer'] .= '<script type="text/javascript"><!--'."\n"
					.'// add a side panel to the current browser instance'."\n"
					.'function addSidePanel() {'."\n"
					.'	// a gecko-based browser: netscape, mozilla, firefox'."\n"
					.'	if((typeof window.sidebar == "object") && (typeof window.sidebar.addPanel == "function")) {'."\n"
					.'		window.sidebar.addPanel("'.strip_tags($context['site_name']).'", "'.$context['url_to_home'].$context['url_to_root'].'panel.php", "");'."\n"
					.'		alert("'.i18n::s('The panel has been added. You may have to ask your browser to make it visible (Ctrl-B for Firefox).').'");'."\n"
					.'	} else {'."\n"
					.'		// internet explorer'."\n"
					.'		if(document.all) {'."\n"
					.'			window.open("'.$context['url_to_home'].$context['url_to_root'].'panel.php?target=_main" ,"_search");'."\n"
					.'		// side panels are not supported'."\n"
					.'		} else {'."\n"
					.'			var rv = alert("'.i18n::s('Your browser does not support side panels. Have you considered to upgrade to Mozilla Firefox?').'");'."\n"
					.'			if(rv)'."\n"
					.'				document.location.href = "http://www.mozilla.org/products/firefox/";'."\n"
					.'		}'."\n"
					.'	}'."\n"
					.'}'."\n"
					.'// -->'."\n"
					.'</script>'."\n";

				// the command to install a bookmaklet into internet explorer
				$context['text'] .= '<p>'.sprintf(i18n::s('If your are running Internet Explorer under Windows, click on the following link to %s triggered on right-click. Accept registry updates, and restart the browser afterwards.'), Skin::build_link('articles/ie_bookmarklet.php', i18n::s('add a contextual bookmarklet'))).'</p>'."\n";

			}

			//
			// extra boxes
			//

			// tools for associates
			if(class_exists('Skin') && Surfer::is_associate()) {

				$box['bar'] = array();
				$box['text'] = '';
				$box['title'] = '';

				$box['title'] = i18n::s('Tools');

				$box['text'] .= '<li>'.Skin::build_link('sections/', i18n::s('Site Map'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('control/populate.php', i18n::s('Content Assistant'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('scripts/stage.php', i18n::s('Upgrade Assistant'), 'shortcut')."</li>\n";

				// the hook for the control panels
				if(is_callable(array('Hooks', 'link_scripts')))
					$box['text'] .= Hooks::link_scripts('control/#tools', 'bullets');

				$box['text'] .= '<li>'.Skin::build_link('help.php', i18n::s('Help'), 'shortcut')."</li>\n";

				// list modules in an extra box
				$context['extra'] .= Skin::build_box($box['title'], '<ul>'.$box['text']."</ul>\n", 'extra');

			}

			// list modules if a skin has been defined
			if(class_exists('Skin')) {

				$box['bar'] = array();
				$box['text'] = '';
				$box['title'] = '';

				$box['title'] = i18n::s('Modules');

				$box['text'] = i18n::s('Most important modules at this site:')."\n<ul>\n";

				if(Surfer::is_associate())
					$box['text'] .= '<li>'.Skin::build_link('actions/', i18n::s('Actions'), 'shortcut')."</li>\n";
				if(Surfer::is_associate())
					$box['text'] .= '<li>'.Skin::build_link('agents/', i18n::s('Agents'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('articles/', i18n::s('Articles'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('categories/', i18n::s('Categories'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('codes/', i18n::s('Codes'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('collections/', i18n::s('Collections'), 'shortcut')."</li>\n";
				if(Surfer::is_associate())
					$box['text'] .= '<li>'.Skin::build_link('comments/', i18n::s('Comments'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('dates/', i18n::s('Dates'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('feeds/', i18n::s('Feeds'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('files/', i18n::s('Files'), 'shortcut')."</li>\n";
				if(Surfer::is_associate())
					$box['text'] .= '<li>'.Skin::build_link('images/', i18n::s('Images'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('letters/', i18n::s('Letters'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('links/', i18n::s('Links'), 'shortcut')."</li>\n";
				if(Surfer::is_associate())
					$box['text'] .= '<li>'.Skin::build_link('locations/', i18n::s('Locations'), 'shortcut')."</li>\n";
				if(Surfer::is_associate())
					$box['text'] .= '<li>'.Skin::build_link('overlays/', i18n::s('Overlays'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('scripts/', i18n::s('Scripts'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('sections/', i18n::s('Sections'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('servers/', i18n::s('Servers'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('services/', i18n::s('Services'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('skins/', i18n::s('Skins'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('smileys/', i18n::s('Smileys'), 'shortcut')."</li>\n";
				if(Surfer::is_associate())
					$box['text'] .= '<li>'.Skin::build_link('tables/', i18n::s('Tables'), 'shortcut')."</li>\n";
				$box['text'] .= '<li>'.Skin::build_link('users/', i18n::s('Users'), 'shortcut')."</li>\n";

				// the hook for the control panels
				if(is_callable(array('Hooks', 'link_scripts')))
					$box['text'] .= Hooks::link_scripts('control/#modules', 'bullets');

				// list modules in an extra box
				$context['extra'] .= Skin::build_box($box['title'], $box['text']."</ul>\n", 'extra');

				// A link to the test page
				$context['extra'] .= Skin::build_box(i18n::s('Test page'), Skin::build_link('control/test.php', i18n::s('Browser and server information'), 'shortcut'), 'navigation', 'test');			}
		}
	}
}

// render the skin
render_skin();

?>