<?php
/**
 * group events by dates
 *
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
Class Layout_dates_as_family extends Layout_interface {

	/**
	 * list dates by dates...
	 *
	 * @param resource the SQL result
	 * @return string the rendered text
	**/
	function &layout(&$result) {
		global $context;

		// empty list
		if(!SQL::count($result)) {
			$output = array();
			return $output;
		}

		// load localized strings
		i18n::bind('dates');

		// flag sections updated recently
		if($context['site_revisit_after'] < 1)
			$context['site_revisit_after'] = 2;
		$dead_line = gmstrftime('%Y-%m-%d %H:%M:%S', mktime(0,0,0,date("m"),date("d")-$context['site_revisit_after'],date("Y")));
		$now = gmstrftime('%Y-%m-%d %H:%M:%S');

		// layout in a table
		$text = Skin::table_prefix();

		// build a list of sections
		$family = '';
		while($item =& SQL::fetch($result)) {

			// change the family
			$this_family = Skin::build_date(substr($item['date_stamp'], 0, 10), 'calendar');
			if($this_family != $family) {
				$family = $this_family;

				$text .= '<tr><td class="family" colspan="2">'.$family.'&nbsp;</td></tr>'."\n";
			}

			// url to view the anchor page
			if(isset($item['anchor']) && preg_match('/article:(\d+)$/', $item['anchor'], $matches)) {
				include_once $context['path_to_root'].'articles/articles.php';
				$url = Articles::get_url($matches[1]);

			// url to view the date
			} else
				$url = Dates::get_url($item['id']);

			// reset everything
			$prefix = $label = $suffix = $icon = '';

			// signal restricted and private dates/articles
			if(!isset($item['publish_date']) || ($item['publish_date'] <= NULL_DATE))
				$prefix .= DRAFT_FLAG;

			// signal restricted and private dates/articles
			if(!isset($item['active']))
				;
			elseif($item['active'] == 'N')
				$prefix .= PRIVATE_FLAG;
			elseif($item['active'] == 'R')
				$prefix .= RESTRICTED_FLAG;

			// flag new dates/articles
			if($item['edit_date'] >= $dead_line)
				$suffix = UPDATED_FLAG.' ';

			// build a valid label
			if(isset($item['title']))
				$label = $item['title'];
			else
				$label = Skin::build_date($item['date_stamp'], 'standalone');

			// indicate the id in the hovering popup
			$hover = i18n::s('Read the page');

			// use the title as a link to the page
			$title = Skin::build_link($url, ucfirst(Codes::beautify(strip_tags($item['title'], '<br><div><img><p><span>'))), 'basic', $hover);

			// also use a clickable thumbnail, if any
			$thumbnail = DECORATED_IMG;
			if($item['thumbnail_url'])
				$thumbnail = Skin::build_link($url, '<img src="'.$item['thumbnail_url'].'" alt="" title="'.encode_field($hover).'" class="left_image"'.EOT, 'basic', $hover);

			// board introduction
			if($item['introduction'])
				$suffix .= '<br style="clear: none;"/>'.Codes::beautify($item['introduction']);

			// this is another row of the output
			$text .= Skin::table_row(array($thumbnail, $prefix.$title.$suffix));

		}

		// end of processing
		SQL::free($result);
		$text .= Skin::table_suffix();
		return $text;
	}
}

?>