<?php
/**
 * one year of dates
 *
 * Accept following invocations:
 * - year.php
 * - year.php/2012
 * - year.php?year=2012
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once '../shared/global.php';
include_once 'dates.php';

// target year
$year = 0;
if(isset($context['arguments'][0]))
	$year = $context['arguments'][0];
elseif(isset($_REQUEST['year']))
	$year = $_REQUEST['year'];
$year = strip_tags($year);
if($year < 1970)
	$year = (int)gmstrftime('%Y');

// root cache id for this page
$root_cache_id = 'dates/year.php/'.$year;

// render a previous version, if any
Cache::pass_through($root_cache_id);

// load localized strings
i18n::bind('dates');

// load the skin
load_skin('dates');

// the title of the page
$context['page_title'] = $year;

// cache between modifications
$cache_id = 'dates/year.php/'.$year;
if(!$text =& Cache::get($cache_id)) {

	// draw every month
	for($month = 1; $month <= 12; $month++)
		$text .= Dates::build_month_calendar($year, $month, 'year');

	// previous year
	$previous = $year-1;

	// next year
	$next = $year+1;

	// neighbours
	$neighbours = array(Dates::get_url($previous, 'year'), $previous,
		Dates::get_url($next, 'year'), $next,
		NULL, NULL);

	// links to display previous and next years
	$text .= Skin::neighbours($neighbours, 'slideshow');

	// save in cache as a global object
	Cache::put($cache_id, $text);

}

// in the main panel
$context['text'] .= $text;

// side bar with the list of most recent pages
$cache_id = 'dates/year.php/'.$year.'#articles_by_date';
if(!$text =& Cache::get($cache_id)) {
	include_once '../articles/articles.php';
	if($items = Articles::list_by_date(0, COMPACT_LIST_SIZE, 'compact')) {
		$text =& Skin::build_box(i18n::s('Recent pages'), Skin::build_list($items, 'compact'), 'extra');
	}
	Cache::put($cache_id, $text, 'articles');
}
$context['extra'] .= $text;

// referrals, if any
if(Surfer::is_associate() || (isset($context['with_referrals']) && ($context['with_referrals'] == 'Y'))) {

	$cache_id = 'dates/year.php/'.$year.'#referrals#';
	if(!$text =& Cache::get($cache_id)) {

		// box content
		include_once '../agents/referrals.php';
		$text = Referrals::list_by_hits_for_url($context['url_to_root_parameter'].'dates/year.php/'.$year.'');

		// in a sidebar box
		if($text)
			$text =& Skin::build_box(i18n::s('Referrals'), $text, 'navigation', 'referrals');

		// save in cache for one hour 60 * 60 = 3600
		Cache::put($cache_id, $text, 'referrals', 3600);

	}

	// in the extra panel
	$context['extra'] .= $text;
}

// render the skin
render_skin($root_cache_id);

?>