<?php
/**
 * the index page for decisions
 *
 * For a comprehensive description of decisions, you should check the database abstraction script
 * at [script]decisions/decisions.php[/script].
 *
 * This page list threads, based on recent decisions.
 * Therefore, this index page is a very convenient place to look for most recent decisions of your community.
 *
 * A list of most recent articles is displayed as a sidebar.
 *
 * Accept following invocations:
 * - index.php (view the 20 top decisions)
 * - index.php/2 (view decisions 41 to 60)
 * - index.php?page=2 (view decisions 41 to 60)
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once '../shared/global.php';
include_once 'decisions.php';

// which page should be displayed
$page = 1;
if(isset($_REQUEST['page']))
	$page = $_REQUEST['page'];
elseif(isset($context['arguments'][0]))
	$page = $context['arguments'][0];
$page = strip_tags($page);

// root cache id for this page
$root_cache_id = 'decisions/index.php/'.$page;

// render a previous version, if any
Cache::pass_through($root_cache_id);

// load localized strings
i18n::bind('decisions');

// load the skin
load_skin('decisions');

// the title of the page
$context['page_title'] = i18n::s('All decision threads');

// count decisions in the database
$stats = Decisions::stat_threads();
if($stats['count'])
	$context['page_menu'] = array_merge($context['page_menu'], array('_count' => sprintf(i18n::ns('1&nbsp;thread', '%d&nbsp;threads', $stats['count']), $stats['count'])));

// navigation commands for decisions, if necessary
if($stats['count'] > THREADS_PER_PAGE) {
	$home = 'decisions/index.php';
	if(isset($context['with_friendly_urls']) && ($context['with_friendly_urls'] == 'Y'))
		$prefix = $home.'/';
	elseif(isset($context['with_friendly_urls']) && ($context['with_friendly_urls'] == 'R'))
		$prefix = $home.'/';
	else
		$prefix = $home.'?page=';
	$context['page_menu'] = array_merge($context['page_menu'], Skin::navigate($home, $prefix, $stats['count'], THREADS_PER_PAGE, $page));
}

// associates may check the database
if(Surfer::is_associate())
	$context['page_menu'] = array_merge($context['page_menu'], array( 'decisions/check.php' => i18n::s('Maintenance') ));

// seek the database
$cache_id = 'decisions/index.php#threads_by_date#'.$page;
if(!$text =& Cache::get($cache_id)) {

	// the first decision to list
	$offset = ($page - 1) * THREADS_PER_PAGE;

	// load the layout to use
	include_once $context['path_to_root'].'articles/articles.php';
	include_once $context['path_to_root'].'articles/layout_articles.php';
	$layout =& new Layout_articles();

	// query the database and layout that stuff
	if(!$text = Decisions::list_threads_by_date($offset, THREADS_PER_PAGE, $layout))
		$context['text'] .= '<p>'.i18n::s('No decision has been recorded yet.').'</p>';

	// we have an array to format
	elseif(@count($text))
		$text =& Skin::build_list($text, 'rows');

	// cache this to speed subsequent queries
	Cache::put($cache_id, $text, 'articles');

}
$context['text'] .= $text;

// side bar with the list of most recent pages
$cache_id = 'decisions/index.php#articles_by_date';
if(!$text =& Cache::get($cache_id)) {
	include_once '../articles/articles.php';
	if($items = Articles::list_by_date(0, COMPACT_LIST_SIZE, 'compact'))
		$text =& Skin::build_box(i18n::s('Recent pages'), Skin::build_list($items, 'compact'), 'extra');
	Cache::put($cache_id, $text, 'articles');
}
$context['extra'] .= $text;

// referrals, if any
if(Surfer::is_associate() || (isset($context['with_referrals']) && ($context['with_referrals'] == 'Y'))) {

	$cache_id = 'decisions/index.php#referrals#';
	if(!$text =& Cache::get($cache_id)) {

		// box content in a sidebar box
		include_once '../agents/referrals.php';
		if($text = Referrals::list_by_hits_for_url($context['url_to_root_parameter'].'decisions/index.php'))
			$text =& Skin::build_box(i18n::s('Referrals'), $text, 'navigation', 'referrals');

		// save in cache for one hour 60 * 60 = 3600
		Cache::put($cache_id, $text, 'referrals', 3600);

	}

	// in the extra panel
	$context['extra'] .= $text;
}

// render the skin
render_skin($root_cache_id);

?>