<?php
/**
 * layout sections as an index page of Yahoo!
 *
 * @todo sort articles by overlay order where applicable
 *
 * With this layout up to three sub-items are listed as well.
 * These can be either sub-sections and/or articles, depending of relative availability of both kind of items.
 *
 * @see sections/sections.php
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
Class Layout_sections_as_yahoo extends Layout_interface {

	/**
	 * list sections
	 *
	 * @param resource the SQL result
	 * @return string the rendered text
	 *
	 * @see skins/layout.php
	**/
	function &layout(&$result) {
		global $context;

		// empty list
		if(!SQL::count($result)) {
			$output = array();
			return $output;
		}

		// load localized strings
		i18n::bind('sections');

		// the number of related items to display
		if(!defined('YAHOO_LIST_SIZE'))
			define('YAHOO_LIST_SIZE', 3);

		// flag sections updated recently
		if($context['site_revisit_after'] < 1)
			$context['site_revisit_after'] = 2;
		$dead_line = strftime('%Y-%m-%d %H:%M:%S', mktime(0,0,0,date("m"),date("d")-$context['site_revisit_after'],date("Y")));
		$now = strftime('%Y-%m-%d %H:%M:%S');

		// we return some text
		$text ='';

		// stack of items
		$items = array();

		// process all items in the list
		$family = '';
		while($item =& SQL::fetch($result)) {

			// change the family
			if($item['family'] != $family) {

				// flush current stack, if any
				if(count($items))
					$text .= Skin::build_list($items, '2-columns');
				$items = array();

				// show the family
				$family = $item['family'];
				$text .= '<h3 class="family">'.$family.'</h3>'."\n";

			}

			// reset the rendering engine between items
			if(is_callable(array('Codes', 'initialize')))
				Codes::initialize(Sections::get_url($item['id'], 'view', $item['nick_name']));

			// initialize variables
			$prefix = $label = $suffix = $icon = '';

			// signal restricted and private sections
			if($item['active'] == 'N')
				$prefix .= PRIVATE_FLAG;
			elseif($item['active'] == 'R')
				$prefix .= RESTRICTED_FLAG;

			// flag sections that are draft, dead, or created or updated very recently
			if($item['activation_date'] >= $now)
				$prefix .= DRAFT_FLAG;
			elseif(($item['expiry_date'] > NULL_DATE) && ($item['expiry_date'] <= $now))
				$prefix .= EXPIRED_FLAG;
			elseif($item['create_date'] >= $dead_line)
				$suffix .= NEW_FLAG;
			elseif($item['edit_date'] >= $dead_line)
				$suffix .= UPDATED_FLAG;

			// details
			$details = array();

			// count related sub-elements
			$related_count = 0;

			// info on related sections
			include_once $context['path_to_root'].'sections/sections.php';
			$stats = Sections::stat_for_anchor('section:'.$item['id']);
			if($stats['count'] > YAHOO_LIST_SIZE)
				$details[] = sprintf(i18n::s('%d&nbsp;sections'), $stats['count']);
			elseif($stats['count'] && Surfer::is_empowered())
				$details[] = sprintf(i18n::ns('1&nbsp;section', '%d&nbsp;sections', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related articles
			include_once $context['path_to_root'].'articles/articles.php';
			$stats = Articles::stat_for_anchor('section:'.$item['id']);
			if($stats['count'] > YAHOO_LIST_SIZE)
				$details[] = sprintf(i18n::s('%d&nbsp;pages'), $stats['count']);
			elseif($stats['count'] && Surfer::is_empowered())
				$details[] = sprintf(i18n::ns('1&nbsp;page', '%d&nbsp;pages', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related files
			include_once $context['path_to_root'].'files/files.php';
			$stats = Files::stat_for_anchor('section:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;file', '%d&nbsp;files', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related links
			include_once $context['path_to_root'].'links/links.php';
			$stats = Links::stat_for_anchor('section:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;link', '%d&nbsp;links', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related comments
			include_once $context['path_to_root'].'comments/comments.php';
			$stats = Comments::stat_for_anchor('section:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;comment', '%d&nbsp;comments', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// rank, for associates
			if(($item['rank'] != 10000) && Surfer::is_empowered())
				$details[] = '{'.$item['rank'].'}';

			// introduction
			if($item['introduction'])
				$suffix .= ' '.strip_tags(Codes::beautify(trim($item['introduction'])), '<a><br><img><li><ol><ul>');

			// append details to the suffix
			if(count($details))
				$suffix .= ' <span class="details">('.implode(', ', $details).')</span>';

			// add a head list of related links
			$details = array();

			// add sub-sections
			include_once $context['path_to_root'].'sections/sections.php';
			if($related =& Sections::list_by_title_for_anchor('section:'.$item['id'], 0, YAHOO_LIST_SIZE, 'compact')) {
				foreach($related as $url => $label) {
					$sub_prefix = $sub_suffix = $sub_hover = '';
					if(is_array($label)) {
						$sub_prefix = $label[0];
						$sub_suffix = $label[2];
						if(@$label[5])
							$sub_hover = $label[5];
						$label = $label[1];
					}
					$details[] = $sub_prefix.Skin::build_link($url, $label, 'basic', $sub_hover).$sub_suffix;
				}
			}

			// add related articles if necessary
			include_once $context['path_to_root'].'articles/articles.php';
			if(preg_match('/\barticles_by_title\b/i', $item['options']))
				$order = 'title';
			elseif(preg_match('/\barticles_by_publication\b/i', $item['options']))
				$order = 'publication';
			elseif(preg_match('/\barticles_by_rating\b/i', $item['options']))
				$order = 'rating';
			else
				$order = 'date';
			if((count($details) < YAHOO_LIST_SIZE) && ($related =& Articles::list_for_anchor_by($order, 'section:'.$item['id'], 0, YAHOO_LIST_SIZE - count($details), 'compact'))) {
				foreach($related as $url => $label) {
					$sub_prefix = $sub_suffix = $sub_hover = '';
					if(is_array($label)) {
						$sub_prefix = $label[0];
						$sub_suffix = $label[2];
						if(@$label[5])
							$sub_hover = $label[5];
						$label = $label[1];
					}
					$details[] = $sub_prefix.Skin::build_link($url, $label, 'basic', $sub_hover).$sub_suffix;
				}
			}

			// add related files if necessary
			include_once $context['path_to_root'].'files/files.php';
			if((count($details) < YAHOO_LIST_SIZE) && ($related = Files::list_by_date_for_anchor('section:'.$item['id'], 0, YAHOO_LIST_SIZE - count($details), 'compact'))) {
				foreach($related as $url => $label) {
					$sub_prefix = $sub_suffix = $sub_hover = '';
					if(is_array($label)) {
						$sub_prefix = $label[0];
						$sub_suffix = $label[2];
						if(@$label[5])
							$sub_hover = $label[5];
						$label = $label[1];
					}
					$details[] = $sub_prefix.Skin::build_link($url, $label, 'basic', $sub_hover).$sub_suffix;
				}
			}

			// add related links if necessary
			include_once $context['path_to_root'].'links/links.php';
			if((count($details) < YAHOO_LIST_SIZE) && ($related = Links::list_by_date_for_anchor('section:'.$item['id'], 0, YAHOO_LIST_SIZE - count($details), 'compact'))) {
				foreach($related as $url => $label) {
					$sub_prefix = $sub_suffix = $sub_hover = '';
					if(is_array($label)) {
						$sub_prefix = $label[0];
						$sub_suffix = $label[2];
						if(@$label[5])
							$sub_hover = $label[5];
						$label = $label[1];
					}
					$details[] = $sub_prefix.Skin::build_link($url, $label, 'basic', $sub_hover).$sub_suffix;
				}
			}

			// give me more
			if(count($details) && ($related_count > YAHOO_LIST_SIZE))
				$details[] = Skin::build_link(Sections::get_url($item['id'], 'view', $item['nick_name']), i18n::s('More').MORE_IMG, 'more', i18n::s('This section contains other information of interest'));

			// layout details
			if(count($details))
				$suffix .= BR.'<span class="small">'.YAHOO_ITEM_PREFIX.implode(YAHOO_ITEM_SUFFIX.YAHOO_ITEM_PREFIX, $details).YAHOO_ITEM_SUFFIX."</span>\n";

			// use the title to label the link
			$label = Skin::strip($item['title'], 50);

			// put the actual icon in the left column
			if(isset($item['thumbnail_url']))
				$icon = $item['thumbnail_url'];

			// the url to view this item
			$url = Sections::get_url($item['id'], 'view', $item['nick_name']);

			// some hovering title for this section
			$hover = i18n::s('View the main page of this section');

			// list all components for this item --use basic link style to avoid prefix or suffix images, if any
			$items[$url] = array($prefix, $label, $suffix, 'basic', $icon, $hover);

		}

		// flush the stack
		if(count($items))
			$text .= Skin::build_list($items, '2-columns');

		// end of processing
		SQL::free($result);
		return $text;
	}

}

?>