<?php
/**
 * the configuration panel for skins
 *
 * This page is called to change rendering parameters of the YACS server.
 *
 * Parameters are grouped as follows:
 * - meta-information - most are used in template files
 * - general rendering options - enable or disable some components
 * - front page - configure the home page as you wish
 * - article pages - parameters specific to articles
 * - image processing
 *
 *
 * Meta-information parameters:
 *
 * [*] [code]site_name[/code] - used everywhere to reference your server.
 * If this parameter is empty, YACS uses the network host name instead (eg, '[code]www.myserver.com[/code]').
 * This parameter is appended to each &lt;title&gt; attribute.
 * It is also used extensively in meta attributes used at the main index page (eg, [code]DC.title[/code]) and
 * in newsfeeds (eg, it is the default value for feed title).
 * Also used at [script]about.php[/script], the description page for your site.
 * Lastly, it is also the default value for subject of new newsletters.
 *
 * [*] [code]site_slogan[/code] - may be displayed at every page, in template.php.
 * During YACS installation a default slogan is proposed.
 * This parameter may be set to an empty string if you don't want any slogan to appear.
 *
 * [*] [code]site_description[/code] - appearing in a &lt;meta&gt; tag of all pages.
 * It is also the default value for channel description in newsfeeds.
 * Also used at [script]about.php[/script], the description page for your site.
 * During YACS installation a default slogan is proposed.
 * This parameter may be set to an empty string if you don't want any description to appear.
 *
 * [*] [code]site_keywords[/code] - appearing in a &lt;meta&gt; tag
 *
 * [*] [code]site_email[/code] - webmaster mail address - warning, this address may be spammed
 *
 * [*] [code]site_copyright[/code] - used in about.php
 *
 * [*] [code]site_owner[/code] - used in about.php
 * @see about.php
 *
 * [*] [code]revisit_after[/code] - appearing in a &lt;meta&gt; tag (default: 7 days)
 *
 * [*] [code]site_position[/code] - the latitute and longitude, separated by a comma.
 * See either [link=GeoTags Search Engine]http://geotags.com/[/link]
 * or [link=Free Geocoding Service for 22 Countries]http://www.travelgis.com/geocode/Default.aspx[/link]
 * for more information.
 *
 * @link http://geotags.com/ GeoTags Search Engine
 * @link http://www.travelgis.com/geocode/Default.aspx Free Geocoding Service for 22 Countries
 *
 * [*] [code]site_head[/code] - inserted as-is in the &lt;head&gt; section of each page
 *
 * [*] [code]site_icon[/code] - the icon for favorites; may be displayed at every page, in template.php
 *
 * [*] [code]site_trailer[/code] - text added at the bottom of every page.
 * This is useful to integrate various javascript libraries (page tracking, etc).
 *
 * General rendering options:
 *
 * [*] [code]site_navigation_maximum[/code] - The maximum number of navigation boxes to display on page side.
 * Default value is 7. This parameter should be used in template file.
 *
 * [*] [code]site_extra_maximum[/code] - The maximum number of extra boxes to display on page side.
 * Default value is 7. This parameter is used in various scripts.
 *
 * [*] [code]with_author_information[/code] - Add links to author pages in lists of articles.
 * By default this information is available only at article pages.
 *
 * [*] [code]pages_without_feeds[/code] - Do not list feeds in sections, categories, and articles
 *
 * [*] [code]pages_without_reference[/code] - To avoid self-referencing
 *
 * [*] [code]pages_without_bookmarklets[/code] - Do not display javascript tools
 *
 * [*] [code]pages_without_history[/code] - To avoid the display of visited pages
 *
 *
 * Parameters for the main panel of the front page:
 *
 * [*] [code]cover_at_home[/code] - Either 'full', 'body' or 'none'.
 * By default YACS displays the title and the content of the cover article at the front page.
 * Select 'body' to mask the title of the cover page.
 * And use 'none' to not display any cover article.
 *
 * [*] [code]sections_count_at_home[/code] - Specify explicitly the number of sections to list at the front page.
 * Default value is 5.
 *
 * [*] [code]sections_at_home[/code] - Either 'menu', or 'map1', or 'map2', or 'none', or some section id.
 * By default YACS lists top sections as a menu bar.
 * Use 'map1' or 'map2' to reflect the site map at the front page.
 * Else put the id of the section you would like to focus on at the front page.
 * Or disable explicit listing of sections if you use a skin that features dynamic tabs, based on sections.
 *
 * [*] [code]flash_at_home[/code] - Insert a Flash object to animate recent titles
 *
 * [*] [code]gadget_boxes_at_home[/code] - Display gadget boxes at the front page.
 * Due to the special layout used for gadget boxes, there is a fixed maximum of 6 gadget boxes.
 *
 * [*] [code]home_layout[/code] - Select the layout to use for articles at the home page.
 * Following values may be selected:
 * - 'alistapart' - Display only the most recent published page.
 * Previous articles may be accessed through a menu.
 * This layout is suitable for small sites with a low activity, maybe with a single section of pages.
 * Implemented at [script]skins/layout_home_articles_as_alistapart.php[/script].
 * - 'boxesandarrows' - List the last ten most recent pages.
 * Previous articles may be accessed through sections, or through the index of articles.
 * This layout is suitable for sites providing several different kinds of information.
 * Implemented at [script]skins/layout_home_articles_as_boxesandarrows.php[/script].
 * - 'compact' - A simple list of titles.
 * Implemented at [script]articles/layout_articles_as_compact.php[/script].
 * - 'daily' (this is also the default value) - Make titles out of publication dates.
 * This layout is suitable for weblogs. It is the default value.
 * Implemented at [script]skins/layout_home_articles_as_daily.php[/script].
 * - 'decorated' - A compact table featuring the introduction and thumbnail.
 * This layout is suitable for sites with a lot of items (gadget boxes, etc.) at the front page.
 * Implemented at [script]articles/layout_articles.php[/script].
 * - 'digg' - To show evidence of rating.
 * Implemented at [script]articles/layout_articles_as_digg.php[/script].
 * - 'newspaper' - Focus on the last published article, and list some articles published previously.
 * This layout is suitable for most sites.
 * Implemented at [script]skins/layout_home_articles_as_newspaper.php[/script].
 * - 'no_articles' - Do not mention recent articles.
 * Use this option to fully customize the front page, for example through some hook.
 * - 'slashdot' - List the last ten most recent pages.
 * Previous articles may be accessed through sections, or through the index of articles.
 * This layout is suitable for sites providing several different kinds of information.
 * Implemented at [script]skins/layout_home_articles_as_slashdot.php[/script].
 * - custom value - useful to implement an external layout.
 * YACS expects to have custom layout ##foo## implemented in ##skins/layout_home_articles_as_foo.php##.
 *
 * [*] [code]articles_count_at_home[/code] - Specify explicitly the number of articles to list at the front page.
 * By default YACS uses the value related to the selected layout.
 *
 * [*] [code]home_with_recent_files[/code] - List most recent files at the front page
 *
 * [*] [code]home_with_recent_links[/code] - List most recent links at the front page
 *
 *
 * Parameters for the extra panel of the front page:
 *
 * [*] [code]featured_at_home[/code] - Either 'static', or 'scroll', or 'rotate', or 'none'.
 * By default YACS lists featured articles as a static list.
 * Use 'scroll' to animate things.
 *
 * [*] [code]featured_count_at_home[/code] - Specify explicitly the number of featured articles to list at the front page.
 *
 * [*] [code]news_at_home[/code] - Either 'static', or 'scroll', or 'rotate', or 'none'.
 * By default YACS lists news articles as a static list.
 * Use 'scroll' to animate the news.
 *
 * [*] [code]news_count_at_home[/code] - Specify explicitly the number of news to list at the front page.
 *
 * [*] [code]home_with_older_articles[/code] - List in a side box articles older that those featured in the
 * main area.
 *
 * [*] [code]home_with_peering_servers[/code] - Display a list of servers that have pinged us recently.
 *
 * [*] [code]home_with_random_articles[/code] - Display a list of random pages.
 *
 * [*] [code]home_with_recent_poll[/code] - List the current poll at the front page
 *
 * [*] [code]home_with_top_articles[/code] - List most read articles at the front page
 *
 * [*] [code]home_with_top_files[/code] - List most fetched files at the front page
 *
 * [*] [code]home_with_top_links[/code] - List most clicked links at the front page
 *
 *
 *
 * Parameters for articles:
 *
 * [*] [code]with_anonymous_side_tools[/code] - To foster registration
 *
 * [*] [code]with_bottom_tools[/code] - Display, or not, conversion tools
 *
 * [*] [code]with_anonymous_bottom_tools[/code] - Spread published information more largely
 *
 *
 * Parameters for image processing:
 *
 * [*] [code]standard_width[/code] - the maximum width, in pixels, for images that have to be resized.
 * Default value is 640 pixels.
 *
 * [*] [code]standard_height[/code] - the maximum height, in pixels, for images that have to be resized.
 * Default value is 640 pixels.
 *
 * [*] [code]avatar_width[/code] - the maximum width, in pixels, for avatars.
 * Default value is 80 pixels.
 *
 * [*] [code]avatar_height[/code] - the maximum height, in pixels, for avatars.
 * Default value is 80 pixels.
 *
 * [*] [code]thumbnail_threshold[/code] - size in bytes that require a thumbnail image.
 * Default value is 20480 bytes.
 *
 * [*] [code]thumbnail_width[/code] - the maximum width, in pixels, for thumbnail images.
 * Default value is 60 pixels.
 *
 * [*] [code]thumbnail_height[/code] - the maximum height, in pixels, for thumbnail images.
 * Default value is 60 pixels.
 *
 * @see images/edit.php
 *
 * [*] [code]thumbnails_without_caption[/code] - If set to 'Y', only a hovering title is provided.
 * Else, which is the default, image title is added to thumbnail as caption.
 *
 *
 * Access to this page is reserved to associates.
 *
 * Configuration information is saved into [code]parameters/skins.include.php[/code].
 * If YACS is prevented to write to the file, it displays parameters to allow for a manual update.
 *
 * The file [code]parameters/skins.include.php.bak[/code] can be used to restore
 * the active configuration before the last change.
 *
 * If the file [code]demo.flag[/code] exists, the script assumes that this instance
 * of YACS runs in demonstration mode.
 * In this mode the edit form is displayed, but parameters are not saved in the configuration file.
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author Christophe Battarel [email]christophe.battarel@altairis.fr[/email]
 * @author GnapZ
 * @tester Pat
 * @tester Natice
 * @tester FabriceV
 * @tester Aleko
 * @tester Guillaume Perez
 * @tester Mark
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
// common definitions and initial processing
include_once '../shared/global.php';

// load the skin
load_skin('skins');

// the path to this page
$context['path_bar'] = array( 'control/' => i18n::s('Control Panel') );

// the title of the page
$context['page_title'] = i18n::s('The configuration panel for the rendering engine');

// not first installation
if(file_exists('../parameters/switch.on') || file_exists('../parameters/switch.off')) {

	// do it again
	if(isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] == 'POST'))
		$context['page_menu'] = array_merge($context['page_menu'], array( 'skins/configure.php' => i18n::s('Edit') ));

	// the back button
	$context['page_menu'] = array_merge($context['page_menu'], array( 'skins/' => i18n::s('Select another skin') ));

}

// anonymous users are invited to log in or to register
if(!Surfer::is_logged())
	Safe::redirect($context['url_to_home'].$context['url_to_root'].'users/login.php?url='.urlencode('skins/configure.php'));

// only associates can proceed
elseif(!Surfer::is_associate()) {
	Safe::header('Status: 403 Forbidden', TRUE, 403);
	Skin::error(i18n::s('You are not allowed to perform this operation.'));

// display the input form
} elseif(isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] != 'POST')) {

	// first installation
	if(!file_exists('../parameters/switch.on') && !file_exists('../parameters/switch.off'))
		$context['text'] .= '<p>'.i18n::s('You can use default values and change these later on. Hit the button at the bottom of the page to move forward.')."</p>\n";

	// load current parameters, if any
	Safe::load('parameters/skins.include.php');

	// the form
	$context['text'] .= '<form method="post" action="'.$context['script_url'].'" id="main_form">';

	// splash message
	$context['text'] .= '<p>'.i18n::s('Click on titles to access various rendering options.')."</p>\n";

	//
	// meta-information
	//

	// site name
	$label = i18n::s('Site name');
	$input = '<input type="text" name="site_name" size="50" value="'.encode_field($context['site_name']).'" maxlength="255"'.EOT;
	$hint = i18n::s('Short and meaningful title, such as "My little big server", used almost everywhere, and appended to the title of every page of this site');
	$fields[] = array($label, $input, $hint);

	// site slogan
	if(!isset($context['site_slogan']))
		$context['site_slogan'] = '-- just do it. and do it right. and make it free. and let it be. and... (your turn)';
	$label = i18n::s('Site slogan');
	$input = '<input type="text" name="site_slogan" size="50" value="'.encode_field($context['site_slogan']).'" maxlength="255"'.EOT;
	$hint = i18n::s('This tag line will be repeated at a number of pages');
	$fields[] = array($label, $input, $hint);

	// site description
	if(!isset($context['site_description']))
		$context['site_description'] = 'YACS Community';
	$label = i18n::s('Site description');
	$input = '<textarea name="site_description" cols="40" rows="2">'.encode_field($context['site_description']).'</textarea>';
	$hint = i18n::s('Up to two lines of text, used in the "description" meta field to help search engines');
	$fields[] = array($label, $input, $hint);

	// keywords
	$label = i18n::s('Keywords');
	$input = '<input type="text" name="site_keywords" size="40" value="'.encode_field(isset($context['site_keywords']) ? $context['site_keywords'] : '').'" maxlength="255"'.EOT;
	$hint = i18n::s('Keywords separated with commas, inserted in the "keyword" meta field for search engines');
	$fields[] = array($label, $input, $hint);

	// site_email address
	if(!isset($context['site_email']) || !$context['site_email'])
		$context['site_email'] = 'unknown_webmaster@acme.heaven';
	$label = i18n::s('Contact mail address');
	$input = '<input type="text" name="site_email" size="40" value="'.encode_field($context['site_email']).'" maxlength="255"'.EOT;
	$hint = sprintf(i18n::s('May be subject to spam attacks; featured in the %s page and in RSS feeds, at least'),
		Skin::build_link('about.php', i18n::s('about'), 'shortcut'));
	$fields[] = array($label, $input, $hint);

	// copyright
	$label = i18n::s('Copyright');
	$input = '<input type="text" name="site_copyright" size="50" value="'.encode_field(isset($context['site_copyright']) ? $context['site_copyright'] : '').'" maxlength="255"'.EOT;
	$hint = i18n::s('Example: "2002-2003, Acme incorporated"; inserted in the "copyright" meta field');
	$fields[] = array($label, $input, $hint);

	// site owner
	$label = i18n::s('Site owner');
	$input = '<input type="text" name="site_owner" size="50" value="'.encode_field(isset($context['site_owner']) ? $context['site_owner'] : '').'" maxlength="255"'.EOT;
	$hint = sprintf(i18n::s('The name of the site owner, e.g. "ACME and company"; Featured in the %s page and in ATOM feeds, at least'),
		Skin::build_link('about.php', i18n::s('about'), 'shortcut'));
	$fields[] = array($label, $input, $hint);

	// revisit after
	if(!isset($context['site_revisit_after']) || !$context['site_revisit_after'])
		$context['site_revisit_after'] = 7;
	$label = i18n::s('Usual delay between updates');
	$input = '<input type="text" name="site_revisit_after" size="4" value="'.encode_field($context['site_revisit_after']).'" maxlength="4"'.EOT.' '.i18n::s('days');
	$hint = i18n::s('Please be realistic here. Used to flag new and updated items. Also featured in "revisit-after" meta field.');
	$fields[] = array($label, $input, $hint);

	// position
	$label = i18n::s('Geographical position');
	$input = '<input type="text" name="site_position" size="40" value="'.encode_field(isset($context['site_position']) ? $context['site_position'] : '').'" maxlength="255"'.EOT;
	$hint = sprintf(i18n::s('Latitude and longitude, separated by a comma, for example: 47.98481,-71.42124, featured as meta fields "geo.position" and "ICBM" at the front page. See %s or %s'),
		Skin::build_link('http://geotags.com/', i18n::s('GeoTags Search Engine'), 'external'),
		Skin::build_link('http://www.travelgis.com/geocode/Default.aspx', i18n::s('Geocoding Service'), 'external'));
	$fields[] = array($label, $input, $hint);

	// head
	$label = i18n::s('Head');
	$input = '<textarea name="site_head" cols="40" rows="2">'.encode_field(isset($context['site_head']) ? $context['site_head'] : '').'</textarea>';
	$hint = i18n::s('Other tags to be inserted into the head section, as meta fields. Please double check generated code to avoid mistakes.  You can use this field to add meta information to your site, that will be used by search engines or software robots. Example: &lt;meta name="dmoz.id" content="put here the dmoz branch for your site"&gt;');
	$fields[] = array($label, $input, $hint);

	// icon
	$label = i18n::s('Icon');
	$input = '<input type="text" name="site_icon" size="40" value="'.encode_field(isset($context['site_icon']) ? $context['site_icon'] : '').'" maxlength="255"'.EOT;
	$hint = i18n::s('The web address of the little image representing your site in favorites or bookmarks. Used to supplement the default "favicon.ico", if any');
	$fields[] = array($label, $input, $hint);

	// trailer
	$label = i18n::s('Trailer');
	$input = '<textarea name="site_trailer" cols="40" rows="2">'.encode_field(isset($context['site_trailer']) ? $context['site_trailer'] : '').'</textarea>';
	$hint = i18n::s('Additional text inserted at the very end of every web page generated by YACS. Use this to integrate page tracking systems, with adequate &lt;script&gt; tags.');
	$fields[] = array($label, $input, $hint);

	// one folded box
	$context['text'] .= Skin::build_box(i18n::s('Meta-information'), Skin::build_form($fields), 'folder');
	$fields = array();

	//
	// general rendering options
	//
	$text = '';

	// maximum number of navigation boxes
	if(!isset($context['site_navigation_maximum']) || !$context['site_navigation_maximum'])
		$context['site_navigation_maximum'] = 7;
	$label = i18n::s('Maximum number of navigation boxes');
	$input = '<input type="text" name="site_navigation_maximum" size="2" value="'.encode_field($context['site_navigation_maximum']).'" maxlength="2"'.EOT;
	$hint = i18n::s('Navigation boxes are displayed on page side, at all pages of the site.');
	$text .= '<p>'.$label.' '.$input.BR.'<span class="details">'.$hint."</span></p>\n";

	// maximum number of extra boxes
	if(!isset($context['site_extra_maximum']) || !$context['site_extra_maximum'])
		$context['site_extra_maximum'] = 7;
	$label = i18n::s('Maximum number of extra boxes');
	$input = '<input type="text" name="site_extra_maximum" size="2" value="'.encode_field($context['site_extra_maximum']).'" maxlength="2"'.EOT;
	$hint = i18n::s('Extra boxes are displayed on the side of pages to which they have been associated.');
	$text .= '<p>'.$label.' '.$input.BR.'<span class="details">'.$hint."</span></p>\n";

	// author information
	$label = i18n::s('Author information');
	$input = '<input type="radio" name="with_author_information" value="N"';
	if(!isset($context['with_author_information']) || ($context['with_author_information'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Hide this information');
	$input .= BR.'<input type="radio" name="with_author_information" value="Y"';
	if(isset($context['with_author_information']) && ($context['with_author_information'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Add links to author profiles in lists of articles');
	$text .= '<p>'.$label.':'.BR.$input."</p>\n";

	// feed
	$label = i18n::s('Side box for RSS feeds');
	$input = '<input type="radio" name="pages_without_feed" value="N"';
	if(!isset($context['pages_without_feed']) || ($context['pages_without_feed'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('List RSS feeds in sections, categories, articles, and user profiles');
	$input .= BR.'<input type="radio" name="pages_without_feed" value="Y"';
	if(isset($context['pages_without_feed']) && ($context['pages_without_feed'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Hide this information');
	$text .= '<p>'.$label.':'.BR.$input."</p>\n";

	// self reference
	$label = i18n::s('Side box for self reference');
	$input = '<input type="radio" name="pages_without_reference" value="N"';
	if(!isset($context['pages_without_reference']) || ($context['pages_without_reference'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Add self-referencing shortcut and web link to visited pages');
	$input .= BR.'<input type="radio" name="pages_without_reference" value="Y"';
	if(isset($context['pages_without_reference']) && ($context['pages_without_reference'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Hide this information');
	$text .= '<p>'.$label.':'.BR.$input."</p>\n";

	// bookmarklets
	$label = i18n::s('Side box for bookmarklets');
	$input = '<input type="radio" name="pages_without_bookmarklets" value="N"';
	if(!isset($context['pages_without_bookmarklets']) || ($context['pages_without_bookmarklets'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Add Javascript bookmarklets to enable further contributions');
	$input .= BR.'<input type="radio" name="pages_without_bookmarklets" value="Y"';
	if(isset($context['pages_without_bookmarklets']) && ($context['pages_without_bookmarklets'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Hide this information');
	$text .= '<p>'.$label.':'.BR.$input."</p>\n";

	// visited pages
	$label = i18n::s('Side box for visited pages');
	$input = '<input type="radio" name="pages_without_history" value="N"';
	if(!isset($context['pages_without_history']) || ($context['pages_without_history'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Remember and list pages visited during a session');
	$input .= BR.'<input type="radio" name="pages_without_history" value="Y"';
	if(isset($context['pages_without_history']) && ($context['pages_without_history'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Do not use this feature');
	$text .= '<p>'.$label.':'.BR.$input."</p>\n";

	// one folded box
	$context['text'] .= Skin::build_box(i18n::s('General rendering options'), $text, 'folder');

	//
	// the front page
	//
	$text = '';

	// the main panel
	$text .= Skin::build_block(i18n::s('The main panel'), 'subtitle');

	// options to display the cover page
	$label = i18n::s('Cover article:');
	$input = '<input type="radio" name="cover_at_home" value="full"';
	if(!isset($context['cover_at_home']) || !preg_match('/(body|none)/', $context['cover_at_home']))
		$input .= ' checked="checked"';
	$input .= ' '.EOT.' '.i18n::s('Display the cover article at the front page.');
	$input .= BR.'<input type="radio" name="cover_at_home" value="body"';
	if(isset($context['cover_at_home']) && ($context['cover_at_home'] == 'body'))
		$input .= ' checked="checked"';
	$input .= ' '.EOT.' '.i18n::s('Display the main part of the cover article, but not the title.');
	$input .= BR.'<input type="radio" name="cover_at_home" value="none"';
	if(isset($context['cover_at_home']) && ($context['cover_at_home'] == 'none'))
		$input .= ' checked="checked"';
	$input .= ' '.EOT.' '.i18n::s('Do not use the cover article at the front page.');
	$text .= '<p>'.$label.BR.$input."</p>\n";

	// sections can be listed as tabs, in the page menu, or as a yahoo-like component
	if(!isset($context['sections_count_at_home']) || ($context['sections_count_at_home'] < 1))
		$context['sections_count_at_home'] = 5;
	$label = sprintf(i18n::s('List up to %s sections:'), '<input type="text" name="sections_count_at_home" value="'.encode_field($context['sections_count_at_home']).'" size="2"'.EOT);
	$input = '<input type="radio" name="sections_at_home" value="menu"';
	if(!isset($context['sections_at_home']) || !$context['sections_at_home'])
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('List sections in the menu bar, right below page title.');
	$input .= BR.'<input type="radio" name="sections_at_home" value="map1"';
	if(isset($context['sections_at_home']) && ($context['sections_at_home'] == 'map1'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('List these sections on one column, in the middle of the front page.');
	$input .= BR.'<input type="radio" name="sections_at_home" value="map2"';
	if(isset($context['sections_at_home']) && ($context['sections_at_home'] == 'map2'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('List these sections on two columns, in the middle of the front page.');
	$input .= BR.'<input type="radio" name="sections_at_home" value="id"';
	if(isset($context['sections_at_home']) && ((int)$context['sections_at_home'] > 0)) {
		$input .= ' checked="checked"';
		$value = $context['sections_at_home'];
	} else {
		$value = 0;
	}
	$input .= EOT.' '.i18n::s('List only section with the following id').' <input type="text" name="section_id_at_home" value="'.encode_field($value).'" size="2"'.EOT;
	$input .= BR.'<input type="radio" name="sections_at_home" value="none"';
	if(isset($context['sections_at_home']) && ($context['sections_at_home'] == 'none'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Do not list sections explicitly -- Useful for skins that feature tabbed sections.');
	$text .= '<p>'.$label.BR.$input."</p>\n";

	// use flash to animate recent pages
	$label = i18n::s('Flash:');
	$input = '<input type="radio" name="flash_at_home" value="N"';
	if(!isset($context['flash_at_home']) || ($context['flash_at_home'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Do not list recent articles in an animated Flash object.');
	$input .= BR.'<input type="radio" name="flash_at_home" value="Y"';
	if(isset($context['flash_at_home']) && ($context['flash_at_home'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.sprintf(i18n::s('List most recent pages in a dynamic and clickable Flash panel. Check this option only if the %s module has been installed. This is the case if you have some text displayed %s.'),
		Skin::build_link('http://ming.sourceforge.net/', 'Ming', 'external'),
		'<a href="'.$context['url_to_root'].'feeds/flash/slashdot.php">'.i18n::s('here').'</a>');
	$text .= '<p>'.$label.BR.$input."</p>\n";

	// gadget boxes
	$label = i18n::s('Gadget boxes:');
	$input = '<input type="radio" name="gadget_boxes_at_home" value="Y"';
	if(!isset($context['gadget_boxes_at_home']) || ($context['gadget_boxes_at_home'] != 'N'))
		$input .= ' checked="checked"';
	include_once '../sections/sections.php';
	$input .= EOT.' '.sprintf(i18n::s('Display up to 6 gadget boxes. Post articles in %s to add more boxes. You can also flag some sections or some categories to list their content in gadget boxes.'), Skin::build_link(Sections::get_url('gadget_boxes'), i18n::s('the section dedicated to gadget boxes'), 'shortcut'));
	$input .= BR.'<input type="radio" name="gadget_boxes_at_home" value="N"';
	if(isset($context['gadget_boxes_at_home']) && ($context['gadget_boxes_at_home'] == 'N'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Do not show gadget boxes at the front page.');
	$text .= '<p>'.$label.BR.$input."</p>\n";

	// splash message for articles layout
	$text .= '<p>'.i18n::s('Following layouts can be used for recent articles:')."</p>\n";

	// default layout is to weblog
	$custom_layout = '';
	if(!isset($context['home_layout']) || !$context['home_layout'])
		$context['home_layout'] = 'daily';
	elseif(!preg_match('/(alistapart|boxesandarrows|compact|daily|decorated|digg|newspaper|no_articles|slashdot)/', $context['home_layout'])) {
		$custom_layout = $context['home_layout'];
		$context['home_layout'] = 'custom';
	}

	// daily
	$text .= '<p><input type="radio" name="home_layout" value="daily"';
	if($context['home_layout'] == 'daily')
		$text .= ' checked="checked"';
	$text .= EOT.'daily - '.i18n::s('For weblogs and blogmarks.')
		.' '.Skin::build_link('skins/layout_home_articles_as_daily.jpg', i18n::s('Preview'), 'help').'</p>';

	// newspaper
	$text .= '<p><input type="radio" name="home_layout" value="newspaper"';
	if($context['home_layout'] == 'newspaper')
		$text .= ' checked="checked"';
	$text .= EOT.'newspaper - '.i18n::s('Focus on the last published article, and on the three articles published previously.')
		.' '.Skin::build_link('skins/layout_home_articles_as_newspaper.jpg', i18n::s('Preview'), 'help').'</p>';

	// boxesandarrows
	$text .= '<p><input type="radio" name="home_layout" value="boxesandarrows"';
	if($context['home_layout'] == 'boxesandarrows')
		$text .= ' checked="checked"';
	$text .= EOT.'boxesandarrows - '.i18n::s('Focus on the last two most recent articles, then list previous pages. Click on article titles to read full text.')
		.' '.Skin::build_link('skins/layout_home_articles_as_boxesandarrows.jpg', i18n::s('Preview'), 'help').'</p>';

	// slashdot
	$text .= '<p><input type="radio" name="home_layout" value="slashdot"';
	if($context['home_layout'] == 'slashdot')
		$text .= ' checked="checked"';
	$text .= EOT.'slashdot - '.i18n::s('List most recent pages equally.')
		.' '.Skin::build_link('skins/layout_home_articles_as_slashdot.jpg', i18n::s('Preview'), 'help').'</p>';

	// digg
	$text .= '<p><input type="radio" name="home_layout" value="digg"';
	if($context['home_layout'] == 'digg')
		$text .= ' checked="checked"';
	$text .= EOT.'digg - '.i18n::s('A decorated list of pages that have been most rated by community members.').'</p>';

	// decorated
	$text .= '<p><input type="radio" name="home_layout" value="decorated"';
	if($context['home_layout'] == 'decorated')
		$text .= ' checked="checked"';
	$text .= EOT.'decorated - '.i18n::s('A decorated list of most recent pages. This layout is suitable for sites with a long cover article at the front page.').'</p>';

	// compact
	$text .= '<p><input type="radio" name="home_layout" value="compact"';
	if($context['home_layout'] == 'compact')
		$text .= ' checked="checked"';
	$text .= EOT.'compact - '.i18n::s('A compact list of most recent pages. This layout is suitable for sites with a lot of items (gadget boxes, etc.) at the front page.').'</p>';

	// alistapart
	$text .= '<p><input type="radio" name="home_layout" value="alistapart"';
	if($context['home_layout'] == 'alistapart')
		$text .= ' checked="checked"';
	$text .= EOT.'alistapart - '.i18n::s('Display only the most recent published page. Previous articles may be accessed through a menu. This layout is suitable for sites with a low number of heavy publications.')
		.' '.Skin::build_link('skins/layout_home_articles_as_alistapart.jpg', i18n::s('Preview'), 'help').'</p>';

	// custom
	$text .= '<p><input type="radio" name="home_layout" value="custom"';
	if($context['home_layout'] == 'custom')
		$text .= ' checked="checked"';
	$text .= EOT.' '.sprintf(i18n::s('Use the customized layout %s'), '<input type="text" name="home_custom_layout" value="'.encode_field($custom_layout).'" size="32"'.EOT).'</p>';

	// no article
	$text .= '<p><input type="radio" name="home_layout" value="no_articles"';
	if($context['home_layout'] == 'no_articles')
		$text .= ' checked="checked"';
	include_once '../sections/sections.php';
	$text .= EOT.sprintf(i18n::s('Do not display recent articles. The layout of the front page is solely based on a %s, on %s, plus %s.'),
		Skin::build_link(Sections::get_url('covers'), i18n::s('cover article'), 'shortcut'),
		Skin::build_link('sections/', i18n::s('the site map'), 'shortcut'),
		Skin::build_link(Sections::get_url('gadget_boxes'), i18n::s('gadget boxes'), 'shortcut')).'</p>';

	// number of entries at the front page
	if(!isset($context['articles_count_at_home']))
		$context['articles_count_at_home'] = '';
	$text .= '<p>'.sprintf(i18n::s('Display %s articles at the front page. Put a number if you wish to override the default value for the selected layout.'), '<input type="text" name="articles_count_at_home" size="2" value="'.encode_field($context['articles_count_at_home']).'" maxlength="4"'.EOT).'</p>';

	// optional components for the main panel of the front page
	$text .= '<p>'.i18n::s('Following elements can be added to the main panel:')."</p>\n";

	// with recent files
	$checked = '';
	if(isset($context['home_with_recent_files']) && ($context['home_with_recent_files'] == 'Y'))
		$checked = 'checked="checked" ';
	$text .= '<p><input type="checkbox" name="home_with_recent_files" value="Y" '.$checked.EOT.' '.i18n::s('Include the list of recent files').'</p>';

	// with recent links
	$checked = '';
	if(isset($context['home_with_recent_links']) && ($context['home_with_recent_links'] == 'Y'))
		$checked = 'checked="checked" ';
	$text .= '<p><input type="checkbox" name="home_with_recent_links" value="Y" '.$checked.EOT.' '.i18n::s('Include the list of recent links').'</p>';

	// the extra panel
	$text .= Skin::build_block(i18n::s('The extra panel'), 'subtitle');

	// featured articles can be either a static or an animated list
	$label = i18n::s('Featured:');
	if(!isset($context['featured_count_at_home']) || ($context['featured_count_at_home'] < 1) || ($context['featured_count_at_home'] > 7))
		$context['featured_count_at_home'] = 5;
	$input = '<input type="radio" name="featured_at_home" value="static"';
	if(!isset($context['featured_at_home']) || !preg_match('/(rotate|scroll|none)/', $context['featured_at_home']))
		$input .= ' checked="checked"';
	$input .= EOT.' '.sprintf(i18n::s('List up to %s featured pages aside.'), '<input type="text" name="featured_count_at_home" value="'.encode_field($context['featured_count_at_home']).'" size="2"'.EOT);
	$input .= BR.'<input type="radio" name="featured_at_home" value="scroll"';
	if(isset($context['featured_at_home']) && ($context['featured_at_home'] == 'scroll'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Similar to the first option, except that displayed information is scrolling.');
	$input .= BR.'<input type="radio" name="featured_at_home" value="rotate"';
	if(isset($context['featured_at_home']) && ($context['featured_at_home'] == 'rotate'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Similar to the first option, except that featured are rotated.');
	$input .= BR.'<input type="radio" name="featured_at_home" value="none"';
	if(isset($context['featured_at_home']) && ($context['featured_at_home'] == 'none'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Do not list featured pages.');
	$text .= '<p>'.$label.BR.$input."</p>\n";

	// news can be either a static or an animated list
	$label = i18n::s('News:');
	if(!isset($context['news_count_at_home']) || ($context['news_count_at_home'] < 1) || ($context['news_count_at_home'] > 7))
		$context['news_count_at_home'] = 5;
	$input = '<input type="radio" name="news_at_home" value="static"';
	if(!isset($context['news_at_home']) || !preg_match('/(rotate|scroll|none)/', $context['news_at_home']))
		$input .= ' checked="checked"';
	$input .= EOT.' '.sprintf(i18n::s('List up to %s news aside.'), '<input type="text" name="news_count_at_home" value="'.encode_field($context['news_count_at_home']).'" size="2"'.EOT);
	$input .= BR.'<input type="radio" name="news_at_home" value="scroll"';
	if(isset($context['news_at_home']) && ($context['news_at_home'] == 'scroll'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Similar to the first option, except that displayed information is scrolling.');
	$input .= BR.'<input type="radio" name="news_at_home" value="rotate"';
	if(isset($context['news_at_home']) && ($context['news_at_home'] == 'rotate'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Similar to the first option, except that news are rotated.');
	$input .= BR.'<input type="radio" name="news_at_home" value="none"';
	if(isset($context['news_at_home']) && ($context['news_at_home'] == 'none'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Do not list news.');
	$text .= '<p>'.$label.BR.$input."</p>\n";

	// optional components in the extra panel
	$text .= '<p>'.i18n::s('Following elements can be added to the extra panel:')."</p>\n";

	// with recent poll
	$checked = '';
	if(isset($context['home_with_recent_poll']) && ($context['home_with_recent_poll'] == 'Y'))
		$checked = 'checked="checked" ';
	$text .= '<p><input type="checkbox" name="home_with_recent_poll" value="Y" '.$checked.EOT.' '.i18n::s('Include the last recent poll').'</p>';

	// with peering servers
	$checked = '';
	if(isset($context['home_with_peering_servers']) && ($context['home_with_peering_servers'] == 'Y'))
		$checked = 'checked="checked" ';
	$text .= '<p><input type="checkbox" name="home_with_peering_servers" value="Y" '.$checked.EOT.' '.i18n::s('Include the list of servers that ping us').'</p>';

	// with older articles
	$checked = '';
	if(isset($context['home_with_older_articles']) && ($context['home_with_older_articles'] == 'Y'))
		$checked = 'checked="checked" ';
	$text .= '<p><input type="checkbox" name="home_with_older_articles" value="Y" '.$checked.EOT.' '.i18n::s('List older articles as well').'</p>';

	// with top articles
	$checked = '';
	if(isset($context['home_with_top_articles']) && ($context['home_with_top_articles'] == 'Y'))
		$checked = 'checked="checked" ';
	$text .= '<p><input type="checkbox" name="home_with_top_articles" value="Y" '.$checked.EOT.' '.i18n::s('Include the list of most read articles').'</p>';

	// with top files
	$checked = '';
	if(isset($context['home_with_top_files']) && ($context['home_with_top_files'] == 'Y'))
		$checked = 'checked="checked" ';
	$text .= '<p><input type="checkbox" name="home_with_top_files" value="Y" '.$checked.EOT.' '.i18n::s('Include the list of most popular files').'</p>';

	// with top links
	$checked = '';
	if(isset($context['home_with_top_links']) && ($context['home_with_top_links'] == 'Y'))
		$checked = 'checked="checked" ';
	$text .= '<p><input type="checkbox" name="home_with_top_links" value="Y" '.$checked.EOT.' '.i18n::s('Include the list of most popular links').'</p>';

	// with random articles
	$checked = '';
	if(isset($context['home_with_random_articles']) && ($context['home_with_random_articles'] == 'Y'))
		$checked = 'checked="checked" ';
	$text .= '<p><input type="checkbox" name="home_with_random_articles" value="Y" '.$checked.EOT.' '.i18n::s('Include a sample of random articles').'</p>';

	// one folded box
	$context['text'] .= Skin::build_box(i18n::s('The front page'), $text, 'folder');

	//
	// handling articles
	//
	$text = '';

	// with details
	$label = i18n::s('Details visibility');
	$input = '<input type="radio" name="content_without_details" value="N"';
	if(!isset($context['content_without_details']) || ($context['content_without_details'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Display detailed information (author, date, ...) on all pages');
	$input .= BR.'<input type="radio" name="content_without_details" value="Y"';
	if(isset($context['content_without_details']) && ($context['content_without_details'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Details are displayed only in sections with option \'with_details\'');
	$text .= '<p>'.$label.':'.BR.$input."</p>\n";

	// side tools
	$text .= '<p>&nbsp;</p><p>'.i18n::s('Side tools (comment, post an image, attach a file or a link) allow community member to contribute easily.')."</p>\n";

	// with anonymous side tools
	$label = i18n::s('Side tools visibility');
	$input = '<input type="radio" name="with_anonymous_side_tools" value="N"';
	if(!isset($context['with_anonymous_side_tools']) || ($context['with_anonymous_side_tools'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Side tools are shown only to members');
	$input .= BR.'<input type="radio" name="with_anonymous_side_tools" value="Y"';
	if(isset($context['with_anonymous_side_tools']) && ($context['with_anonymous_side_tools'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Allow anonymous surfers to trigger side tools as well (recommended on intranets)');
	$text .= '<p>'.$label.':'.BR.$input."</p>\n";

	// bottom tools
	$text .= '<p>&nbsp;</p><p>'.i18n::s('Bottom tools (pdf, palm, word, print) are used to export or to convert pages to an alternate format or media.')."</p>\n";

	// with tools
	$label = i18n::s('Bottom tools global control');
	$input = '<input type="radio" name="with_bottom_tools" value="N"';
	if(!isset($context['with_bottom_tools']) || ($context['with_bottom_tools'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Bottom tools are displayed only in sections with option \'with_bottom_tools\'');
	$input .= BR.'<input type="radio" name="with_bottom_tools" value="Y"';
	if(isset($context['with_bottom_tools']) && ($context['with_bottom_tools'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Display bottom tools on all pages');
	$text .= '<p>'.$label.':'.BR.$input."</p>\n";

	// with anonymous bottom tools
	$label = i18n::s('Bottom tools visibility');
	$input = '<input type="radio" name="with_anonymous_bottom_tools" value="N"';
	if(!isset($context['with_anonymous_bottom_tools']) || ($context['with_anonymous_bottom_tools'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Bottom tools are shown only to members');
	$input .= BR.'<input type="radio" name="with_anonymous_bottom_tools" value="Y"';
	if(isset($context['with_anonymous_bottom_tools']) && ($context['with_anonymous_bottom_tools'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Allow anonymous surfers to use bottom tools as well (recommended on intranets)');
	$text .= '<p>'.$label.':'.BR.$input."</p>\n";

	// one folded box
	$context['text'] .= Skin::build_box(i18n::s('Article pages'), $text, 'folder');

	//
	// handling images
	//

	// standard width
	if(!isset($context['standard_width']) || !$context['standard_width'])
		$context['standard_width'] = 640;
	$label = i18n::s('Standard width');
	$input = '<input type="text" name="standard_width" size="5" value="'.encode_field($context['standard_width']).'" maxlength="5"'.EOT;
	$hint = i18n::s('The maximum width for uploaded photos');
	$fields[] = array($label, $input, $hint);

	// standard heigth
	if(!isset($context['standard_height']) || !$context['standard_height'])
		$context['standard_height'] = 640;
	$label = i18n::s('Standard height');
	$input = '<input type="text" name="standard_height" size="5" value="'.encode_field($context['standard_height']).'" maxlength="5"'.EOT;
	$hint = i18n::s('The maximum height for uploaded photos');
	$fields[] = array($label, $input, $hint);

	// avatar width
	if(!isset($context['avatar_width']) || !$context['avatar_width'])
		$context['avatar_width'] = 80;
	$label = i18n::s('Avatar width');
	$input = '<input type="text" name="avatar_width" size="5" value="'.encode_field($context['avatar_width']).'" maxlength="5"'.EOT;
	$hint = i18n::s('The maximum number of pixels for the width of avatars');
	$fields[] = array($label, $input, $hint);

	// avatar heigth
	if(!isset($context['avatar_height']) || !$context['avatar_height'])
		$context['avatar_height'] = 80;
	$label = i18n::s('Avatar height');
	$input = '<input type="text" name="avatar_height" size="5" value="'.encode_field($context['avatar_height']).'" maxlength="5"'.EOT;
	$hint = i18n::s('The maximum number of pixels for the height of avatars');
	$fields[] = array($label, $input, $hint);

	// thumbnail threshold
	if(!isset($context['thumbnail_threshold']) || !$context['thumbnail_threshold'])
		$context['thumbnail_threshold'] = 20480;
	$label = i18n::s('Thumbnail threshold');
	$input = '<input type="text" name="thumbnail_threshold" size="10" value="'.encode_field($context['thumbnail_threshold']).'" maxlength="10"'.EOT;
	$hint = i18n::s('The number of bytes to trigger the creation of a thumbnail');
	$fields[] = array($label, $input, $hint);

	// thumbnail width
	if(!isset($context['thumbnail_width']) || !$context['thumbnail_width'])
		$context['thumbnail_width'] = 60;
	$label = i18n::s('Thumbnail width');
	$input = '<input type="text" name="thumbnail_width" size="5" value="'.encode_field($context['thumbnail_width']).'" maxlength="5"'.EOT;
	$hint = i18n::s('The maximum number of pixels for the width of thumbnails');
	$fields[] = array($label, $input, $hint);

	// thumbnail heigth
	if(!isset($context['thumbnail_height']) || !$context['thumbnail_height'])
		$context['thumbnail_height'] = 60;
	$label = i18n::s('Thumbnail height');
	$input = '<input type="text" name="thumbnail_height" size="5" value="'.encode_field($context['thumbnail_height']).'" maxlength="5"'.EOT;
	$hint = i18n::s('The maximum number of pixels for the height of thumbnails');
	$fields[] = array($label, $input, $hint);

	// thumbnail caption
	$label = i18n::s('Thumbnail caption');
	$input = '<input type="radio" name="thumbnails_without_caption" value="N"';
	if(!isset($context['thumbnails_without_caption']) || ($context['thumbnails_without_caption'] != 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Use titles as captions below thumbnail images');
	$input .= BR.'<input type="radio" name="thumbnails_without_caption" value="Y"';
	if(isset($context['thumbnails_without_caption']) && ($context['thumbnails_without_caption'] == 'Y'))
		$input .= ' checked="checked"';
	$input .= EOT.' '.i18n::s('Only provide a hovering title, but no caption');
	$fields[] = array($label, $input);

	// one folded box
	$context['text'] .= Skin::build_box(i18n::s('Images processing'), '<p>'.i18n::s('Configure the handling of images. YACS uses the GD module of PHP to automatically resize submitted images when they are very large, and to build thumbnail images as well.')."</p>\n".Skin::build_form($fields), 'folder');
	$fields = array();

	// the submit button
	$context['text'] .= '<p>'.Skin::build_submit_button(i18n::s('Submit'), i18n::s('Press [s] to submit data'), 's').'</p>'."\n";

	// end of the form
	$context['text'] .= '</form>';

// no modifications in demo mode
} elseif(file_exists($context['path_to_root'].'parameters/demo.flag')) {

	// remind the surfer
	$context['text'] .= '<p>'.i18n::s('This instance of YACS runs in demonstration mode. For security reasons configuration parameters cannot be changed in this mode.')."</p>\n";

// save updated parameters
} else {

	// ensure we have a valid revisit_after
	if(!isset($context['revisit_after']) || (intval($context['revisit_after']) < 1))
		$context['revisit_after'] = 1;

	// ensure we have a valid layout for the home page
	if(!isset($_REQUEST['home_layout']) || !$_REQUEST['home_layout'] || !preg_match('/(alistapart|boxesandarrows|compact|custom|daily|decorated|digg|newspaper|no_articles|slashdot)/', $_REQUEST['home_layout']))
		$_REQUEST['home_layout'] = 'daily';
	elseif($_REQUEST['home_layout'] == 'custom') {
		if(isset($_REQUEST['home_custom_layout']) && $_REQUEST['home_custom_layout'])
			$_REQUEST['home_layout'] = basename(strip_tags($_REQUEST['home_custom_layout']));
		else
			$_REQUEST['home_layout'] = 'daily';
	}

	// backup the old version
	Safe::unlink($context['path_to_root'].'parameters/skins.include.php.bak');
	if(Safe::rename($context['path_to_root'].'parameters/skins.include.php', $context['path_to_root'].'parameters/skins.include.php.bak'))
		$context['text'] .= '<p>'.sprintf(i18n::s('%s has been renamed with a \'.bak\' extension'), 'parameters/skins.include.php')."</p>\n";

	// build the new configuration file
	$content = '<?php'."\n"
		.'// This file has been created by the configuration script skins/configure.php'."\n"
		.'// on '.gmdate("F j, Y, g:i a").' GMT, for '.Surfer::get_name().'. Please do not modify it manually.'."\n"
		.'global $context;'."\n"
		.'$context[\'site_name\']=\''.addcslashes($_REQUEST['site_name'], "\\'")."';\n"
		.'$context[\'site_description\']=\''.addcslashes($_REQUEST['site_description'], "\\'")."';\n"
		.'$context[\'site_keywords\']=\''.addcslashes($_REQUEST['site_keywords'], "\\'")."';\n"
		.'$context[\'site_owner\']=\''.addcslashes($_REQUEST['site_owner'], "\\'")."';\n"
		.'$context[\'site_email\']=\''.addcslashes($_REQUEST['site_email'], "\\'")."';\n"
		.'$context[\'site_copyright\']=\''.addcslashes($_REQUEST['site_copyright'], "\\'")."';\n"
		.'$context[\'site_icon\']=\''.addcslashes($_REQUEST['site_icon'], "\\'")."';\n"
		.'$context[\'site_head\']=\''.addcslashes(str_replace("\r", '', $_REQUEST['site_head']), "\\'")."';\n"
		.'$context[\'site_revisit_after\']=\''.addcslashes($_REQUEST['site_revisit_after'], "\\'")."';\n"
		.'$context[\'site_slogan\']=\''.addcslashes($_REQUEST['site_slogan'], "\\'")."';\n"
		.'$context[\'site_position\']=\''.addcslashes($_REQUEST['site_position'], "\\'")."';\n"
		.'$context[\'site_trailer\']=\''.addcslashes($_REQUEST['site_trailer'], "\\'")."';\n"
		.'$context[\'home_layout\']=\''.addcslashes($_REQUEST['home_layout'], "\\'")."';\n";
	if(isset($_REQUEST['articles_count_at_home']) && intval($_REQUEST['articles_count_at_home']))
		$content .= '$context[\'articles_count_at_home\']=\''.intval($_REQUEST['articles_count_at_home'])."';\n";
	if(isset($_REQUEST['content_without_details']))
		$content .= '$context[\'content_without_details\']=\''.addcslashes($_REQUEST['content_without_details'], "\\'")."';\n";
	if(isset($_REQUEST['cover_at_home']))
		$content .= '$context[\'cover_at_home\']=\''.addcslashes($_REQUEST['cover_at_home'], "\\'")."';\n";
	if(isset($_REQUEST['flash_at_home']))
		$content .= '$context[\'flash_at_home\']=\''.addcslashes($_REQUEST['flash_at_home'], "\\'")."';\n";
	if(isset($_REQUEST['gadget_boxes_at_home']))
		$content .= '$context[\'gadget_boxes_at_home\']=\''.addcslashes($_REQUEST['gadget_boxes_at_home'], "\\'")."';\n";
	if(isset($_REQUEST['home_with_older_articles']))
		$content .= '$context[\'home_with_older_articles\']=\''.addcslashes($_REQUEST['home_with_older_articles'], "\\'")."';\n";
	if(isset($_REQUEST['home_with_peering_servers']))
		$content .= '$context[\'home_with_peering_servers\']=\''.addcslashes($_REQUEST['home_with_peering_servers'], "\\'")."';\n";
	if(isset($_REQUEST['home_with_random_articles']))
		$content .= '$context[\'home_with_random_articles\']=\''.addcslashes($_REQUEST['home_with_random_articles'], "\\'")."';\n";
	if(isset($_REQUEST['home_with_recent_files']))
		$content .= '$context[\'home_with_recent_files\']=\''.addcslashes($_REQUEST['home_with_recent_files'], "\\'")."';\n";
	if(isset($_REQUEST['home_with_recent_links']))
		$content .= '$context[\'home_with_recent_links\']=\''.addcslashes($_REQUEST['home_with_recent_links'], "\\'")."';\n";
	if(isset($_REQUEST['home_with_recent_poll']))
		$content .= '$context[\'home_with_recent_poll\']=\''.addcslashes($_REQUEST['home_with_recent_poll'], "\\'")."';\n";
	if(isset($_REQUEST['home_with_top_articles']))
		$content .= '$context[\'home_with_top_articles\']=\''.addcslashes($_REQUEST['home_with_top_articles'], "\\'")."';\n";
	if(isset($_REQUEST['home_with_top_files']))
		$content .= '$context[\'home_with_top_files\']=\''.addcslashes($_REQUEST['home_with_top_files'], "\\'")."';\n";
	if(isset($_REQUEST['home_with_top_links']))
		$content .= '$context[\'home_with_top_links\']=\''.addcslashes($_REQUEST['home_with_top_links'], "\\'")."';\n";
	if(isset($_REQUEST['featured_at_home']))
		$content .= '$context[\'featured_at_home\']=\''.addcslashes($_REQUEST['featured_at_home'], "\\'")."';\n";
	if(isset($_REQUEST['featured_count_at_home']))
		$content .= '$context[\'featured_count_at_home\']=\''.addcslashes($_REQUEST['featured_count_at_home'], "\\'")."';\n";
	if(isset($_REQUEST['news_at_home']))
		$content .= '$context[\'news_at_home\']=\''.addcslashes($_REQUEST['news_at_home'], "\\'")."';\n";
	if(isset($_REQUEST['news_count_at_home']))
		$content .= '$context[\'news_count_at_home\']=\''.addcslashes($_REQUEST['news_count_at_home'], "\\'")."';\n";
	if(isset($_REQUEST['pages_without_bookmarklets']))
		$content .= '$context[\'pages_without_bookmarklets\']=\''.addcslashes($_REQUEST['pages_without_bookmarklets'], "\\'")."';\n";
	if(isset($_REQUEST['pages_without_history']))
		$content .= '$context[\'pages_without_history\']=\''.addcslashes($_REQUEST['pages_without_history'], "\\'")."';\n";
	if(isset($_REQUEST['pages_without_reference']))
		$content .= '$context[\'pages_without_reference\']=\''.addcslashes($_REQUEST['pages_without_reference'], "\\'")."';\n";
	if(isset($_REQUEST['sections_at_home'])) {
		if($_REQUEST['sections_at_home'] == 'id') {
			if(isset($_REQUEST['section_id_at_home']) && ((int)$_REQUEST['section_id_at_home'] > 0))
				$_REQUEST['sections_at_home'] = $_REQUEST['section_id_at_home'];
			else
				$_REQUEST['sections_at_home'] = 'none';
		}
		$content .= '$context[\'sections_at_home\']=\''.addcslashes($_REQUEST['sections_at_home'], "\\'")."';\n";
	}
	if(isset($_REQUEST['sections_count_at_home']))
		$content .= '$context[\'sections_count_at_home\']=\''.addcslashes($_REQUEST['sections_count_at_home'], "\\'")."';\n";
	if(isset($_REQUEST['site_extra_maximum']) && intval($_REQUEST['site_extra_maximum']))
		$content .= '$context[\'site_extra_maximum\']=\''.intval($_REQUEST['site_extra_maximum'])."';\n";
	if(isset($_REQUEST['site_navigation_maximum']) && intval($_REQUEST['site_navigation_maximum']))
		$content .= '$context[\'site_navigation_maximum\']=\''.intval($_REQUEST['site_navigation_maximum'])."';\n";
	if(isset($_REQUEST['thumbnails_without_caption']))
		$content .= '$context[\'thumbnails_without_caption\']=\''.addcslashes($_REQUEST['thumbnails_without_caption'], "\\'")."';\n";
	if(isset($_REQUEST['with_anonymous_bottom_tools']))
		$content .= '$context[\'with_anonymous_bottom_tools\']=\''.addcslashes($_REQUEST['with_anonymous_bottom_tools'], "\\'")."';\n";
	if(isset($_REQUEST['pages_without_feed']))
		$content .= '$context[\'pages_without_feed\']=\''.addcslashes($_REQUEST['pages_without_feed'], "\\'")."';\n";
	if(isset($_REQUEST['with_anonymous_side_tools']))
		$content .= '$context[\'with_anonymous_side_tools\']=\''.addcslashes($_REQUEST['with_anonymous_side_tools'], "\\'")."';\n";
	if(isset($_REQUEST['with_bottom_tools']))
		$content .= '$context[\'with_bottom_tools\']=\''.addcslashes($_REQUEST['with_bottom_tools'], "\\'")."';\n";
	$content .= '$context[\'standard_width\']=\''.addcslashes($_REQUEST['standard_width'], "\\'")."';\n"
		.'$context[\'standard_height\']=\''.addcslashes($_REQUEST['standard_height'], "\\'")."';\n"
		.'$context[\'avatar_width\']=\''.addcslashes($_REQUEST['avatar_width'], "\\'")."';\n"
		.'$context[\'avatar_height\']=\''.addcslashes($_REQUEST['avatar_height'], "\\'")."';\n"
		.'$context[\'thumbnail_threshold\']=\''.addcslashes($_REQUEST['thumbnail_threshold'], "\\'")."';\n"
		.'$context[\'thumbnail_width\']=\''.addcslashes($_REQUEST['thumbnail_width'], "\\'")."';\n"
		.'$context[\'thumbnail_height\']=\''.addcslashes($_REQUEST['thumbnail_height'], "\\'")."';\n";
	if(isset($_REQUEST['with_author_information']))
		$content .= '$context[\'with_author_information\']=\''.addcslashes($_REQUEST['with_author_information'], "\\'")."';\n";
	$content .= '?>'."\n";

	// update the parameters file
	if(!Safe::file_put_contents('parameters/skins.include.php', $content)) {

		Skin::error(i18n::s('ERROR: Impossible to write to the file parameters/skins.include.php. The configuration has not been saved.'));

		// allow for a manual update
		$context['text'] .= '<p style="text-decoration: blink;">'.i18n::s('To actually change the configuration, please copy and paste following lines by yourself in file parameters/skins.include.php:')."</p>\n";

	// job done
	} else {

		$context['text'] .= '<p>'.i18n::s('The following configuration has been saved into the file parameters/skins.include.php.')."</p>\n";

		// first installation
		if(!file_exists('../parameters/switch.on') && !file_exists('../parameters/switch.off'))
			$context['text'] .= '<p>'.i18n::s('Review provided information and go to the bottom of the page to move forward.')."</a></p>\n";

		// purge the cache
		Cache::clear();

		// remember the change
		$label = sprintf(i18n::c('%s has been updated'), 'parameters/skins.include.php');
		$description = $context['url_to_home'].$context['url_to_root'].'skins/configure.php';
		Logger::remember('skins/configure.php', $label, $description);
	}

	// display updated parameters
	$context['text'] .= Skin::build_box(i18n::s('Configuration parameters'), Safe::highlight_string($content), 'folder');

	// first installation
	if(!file_exists('../parameters/switch.on') && !file_exists('../parameters/switch.off')) {
		$context['text'] .= '<form method="get" action="../control/" id="main_form">'."\n"
			.'<p>'.Skin::build_submit_button(i18n::s('Switch the server on')).'</p>'."\n"
			.'</form>'."\n";

	// ordinary follow-up commands
	} else {

		// what's next?
		$context['text'] .= '<p>'.i18n::s('What do you want to do now?')."</p>\n";

		// follow-up menu
		$menu = array();

		// offer to change it again
		$menu = array_merge($menu, array( 'skins/configure.php' => i18n::s('Edit again') ));

		// back to the control panel
		$menu = array_merge($menu, array( 'control/' => i18n::s('Go to the Control Panel') ));

		// display follow-up commands
		$context['text'] .= Skin::build_list($menu, 'menu_bar');

	}

}

// render the skin
render_skin();

?>