<?php
/**
 * the library of smileys
 *
 * Smileys are these little icons that we are using to briefly report on our feeling.
 *
 * To add a new smiley on this system, you will have:
 * - to prepare a new icon file
 * - to name it
 * - to update smileys.php to bind the name to the file
 * - to update the page below to show it working
 *
 * @link smileys/index.php The current list of supported smileys on this server
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @author Lucrecius
 * @tester Agnes
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

class Smileys {

	/**
	 * called from within a preg_replace_callback() in Smileys::render_smileys()
	 *
	 */
	function parse_match($matches) {
		global $context;

		// target tag
		$tag = $matches[1];

		// ensure a file exists for this tag
		if(!file_exists($context['path_to_root'].'skins/images/smileys/'.$tag.'.gif'))
			return $matches[0];

		// make a valid reference to an image
		return ' <img src="'.$context['url_to_root'].'skins/images/smileys/'.$tag.'.gif" alt="'.$tag.'" /> ';

	}

	/**
	 * transform some text to load related images
	 *
	 * @param string the input text
	 * @return the tansformed text
	 */
	function render_smileys($text) {
		global $context;

		// sanity check
		if(!trim($text))
			return '';

		// the list of codes to be interpreted --initialize only once
		static $codes;
		if(!isset($codes)) {
			$codes = array(
				'/(^>:\(| >:\(|>:-\()/',
				'/(^:D| :D|:-D)/',
				'/(^:\'\(| :\'\(|:\'-\()/',
				'/(^8\)| 8\)|8-\))/',
				'/(^:\(| :\(|:-\()/',
				'/(\?\?\?)/',
				'/(^:\[| :\[|:-\[)/',
				'/:blush:/',
				'/(^:X| :X|:-X)/',
				'/(^:P| :P|:-P)/',
				'/:medal:/',
				'/:half_medal:/',
				'/(^::\)| ::\)|::-\))/',	// ::) before :)
				'/(^:\)| :\)|:-\))/',
				'/(^:o| :o|:-o)/',
				'/(^:\/| :\/|:-\/)/',
				'/(^;\)| ;\)|;-\))/',
				'/:party:/',
				'/:\*:/',
				'/(^:\*| :\*|:-\*)/',
				'/(^:\+| :\+|:up:)/',
				'/(^:-| :-|:down:)/',
				'/(^:\?!|:\?!)/',
				'/(^:\?2| :\?2)/',
				'/(^:\?| :\?)/',
				'/(^:!2| :!2)/',
				'/(^:!| :!)/');
		}

		// the related <img> html elements --initialize only once
		static $images;
		if(!isset($images)) {
			$prefix = ' <img src="'.$context['url_to_root'].'skins/images/smileys/';
			$suffix = '" alt=""'.EOT.' ';
			$images = array(
				$prefix.'angry.gif'.$suffix,
				$prefix.'cheesy.gif'.$suffix,
				$prefix.'cry.gif'.$suffix,
				$prefix.'cool.gif'.$suffix,
				$prefix.'frown.gif'.$suffix,
				$prefix.'confused.gif'.$suffix,
				$prefix.'embarassed.gif'.$suffix,
				$prefix.'blushing.gif'.$suffix,
				$prefix.'sealed.gif'.$suffix,
				$prefix.'tongue.gif'.$suffix,
				$prefix.'medal_full.gif'.$suffix,
				$prefix.'medal_half.gif'.$suffix,
				$prefix.'rolleyes.gif'.$suffix,
				$prefix.'smile.gif'.$suffix,
				$prefix.'shocked.gif'.$suffix,
				$prefix.'undecided.gif'.$suffix,
				$prefix.'winkgrin.gif'.$suffix,
				$prefix.'partygirl.gif'.$suffix,
				$prefix.'star.gif'.$suffix,
				$prefix.'kiss.gif'.$suffix,
				$prefix.'thumbsup.gif'.$suffix,
				$prefix.'thumbsdown.gif'.$suffix,
				$prefix.'idea.gif'.$suffix,
				$prefix.'question2.gif'.$suffix,
				$prefix.'question.gif'.$suffix,
				$prefix.'exclamation2.gif'.$suffix,
				$prefix.'exclamation.gif'.$suffix );
		}

		// ensure we have enough processing time
		Safe::set_time_limit(30);

		// process dotted smileys
		$text = preg_replace($codes, $images,  $text);

		// process any image file
		return preg_replace_callback('/:([\w_]+):/', array('smileys', 'parse_match'),  $text);
	}
}
?>