<?php
/**
 * track one item
 *
 * This page is triggered by logged users, to managed items that they manage.
 *
 * Accept following invocations:
 * - track.php/section/2 (track section #2)
 * - track.php?anchor=section:2 (track section #2)
 * - track.php/article/12 (track article #12)
 * - track.php?anchor=article:12 (track article #12)
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once '../shared/global.php';

// tracked item
$track = NULL;
if(isset($_REQUEST['section']))
	$track = 'section:'.$_REQUEST['section'];
elseif(isset($_REQUEST['article']))
	$track = 'article:'.$_REQUEST['article'];
elseif(isset($_REQUEST['anchor']))
	$track = $_REQUEST['anchor'];
elseif(isset($context['arguments']))
	$track = $context['arguments'][0].':'.$context['arguments'][1];
$track = strip_tags($track);

// get the item from the database
$item = NULL;
$anchor = get_anchor($track);
if(is_object($anchor))
	$item = $anchor->item;

// load localized strings
i18n::bind('users');

// load the skin
load_skin('users');

// the path to this page
$context['path_bar'] = array( 'users/' => i18n::s('Users') );
if(Surfer::is_logged())
	$context['path_bar'] = array_merge($context['path_bar'], array( Users::get_url(Surfer::get_id(), 'view', Surfer::get_name()) => Surfer::get_name() ));

// the title of the page
$context['page_title'] = i18n::s('Watch list update');

// not found
if(!$item['id'])
	Skin::error(i18n::s('No page has the provided id.').' ('.$track.')');

// operation is restricted to logged users
elseif(!Surfer::is_logged())
	Skin::error(i18n::s('You are not allowed to perform this operation.'));

// toggle membership status
elseif($error = Members::toggle($track, 'user:'.Surfer::get_id())) {
	Skin::error($error);

// post-processing tasks
} else {

	// successful operation reflected into page title
	$context['page_title'] = i18n::s('Your watch list has been successfully updated');

	// reference the anchor page
	if(is_object($anchor) && $anchor->is_viewable())
		$context['text'] .= '<p>'.sprintf(i18n::s('Referenced page: %s'), Skin::build_link($anchor->get_url(), $anchor->get_title()))."</p>\n";

	// follow-up commands
	$menu = array();
	if(is_object($anchor) && $anchor->is_viewable())
		$menu = array($anchor->get_url() => i18n::s('Back to the referenced page'));

	// the page now appears in the watch list
	if(Members::check($track, 'user:'.Surfer::get_id())) {
		$context['text'] .= '<p>'.i18n::s('The page has been added to your watch list. You will receive electronic messages to warn you on each future update.')."</p>\n";

		$menu = array_merge($menu, array('users/track.php?anchor='.urlencode($track) => i18n::s('I have changed my mind, forget this page')));

	// the page has been suppressed from the watch list
	} else {
		$context['text'] .= '<p>'.i18n::s('The page has been deleted from your watch list. You won\'t receive any message about it anymore.')."</p>\n";

		$menu = array_merge($menu, array('users/track.php?anchor='.urlencode($track) => i18n::s('I have changed my mind, watch this page')));

	}

	$context['text'] .= '<p>'.i18n::s('Please note that pages automatically appear in your watch list if you modify them either directly or by posting new pieces of information (e.g., comment, image, or file).')."</p>\n";

	if(Surfer::is_logged())
		$menu = array_merge($menu, array(Users::get_url(Surfer::get_id()).'#watch_list' => i18n::s('View my watch list')));

	// splash message
	$context['text'] .= '<p>'.i18n::s('What do you want to do now?').'</p>';

	$context['text'] .= Skin::build_list($menu, 'menu_bar');

}

// failed operation
if(count($context['error']))
	$context['text'] .= '<p>'.i18n::s('Operation has failed').'</p>';

// render the skin
render_skin();

?>